@@ -1,4 +1,4 @@
-## Copyright (C) 2009,2010 Philip
+## Copyright (C) 2009,2010,2011 Philip Nienhuis
 ## 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -27,6 +27,10 @@
 ## 2010-03-17 Simplified argument list, only row + column needed
 ## 2010-09-27 Made error message more comprehensible
 ## 2010-10-11 Added check for row range
+## 2011-04-21 Added tests
+## 2011-04-30 Simplified column name computation
+## 2011-12-17 Bugfix for wrong column address if column equals multiple of 26
+## 2011-12-18 Added tests for multiple-of-26 cases
 
 function [ celladdress ] = calccelladdress (row, column)
 
@@ -34,18 +38,52 @@ function [ celladdress ] = calccelladdress (row, column)
 
 	if (column > 18278 || column < 1) error ("Specified column out of range (1..18278)"); endif
 	if (row > 1048576 || row < 1), error ('Specified row out of range (1..1048576)'); endif
-	rem1 = rem ((column-1), 26);
-	str = char (rem1 + 'A');								# A-Z; rightmost digit
-	if (column > 26 && column < 703)						# AA-ZZ
-		tmp = char (floor(column - rem1) / 26 - 1 + 'A');	# Leftmost digit
-		str = [tmp str];
-	elseif (column > 702 && column < 18279)					# AAA-ZZZ
-		rem2 = rem ((column - 26 - rem1) - 1, 676);
-		str2 = char (rem2 / 26 + 'A');						# Middle digit
-		column = column -  rem2 - rem1;
-		str3 = char (column / 676 - 1 + 'A');				# Leftmost digit
-		str = [str3 str2 str];
-	endif	
+
+	str = '';
+	while (column > 0.01)
+		rmd = floor ((column - 1) / 26);
+		str = [char(column - rmd * 26 + 'A' - 1) str];
+		column = rmd;
+	endwhile
+
 	celladdress = sprintf ("%s%d", str, row);
 
-endfunction
\ No newline at end of file
+endfunction
+
+%!test
+%! a = calccelladdress (1, 1);
+%! assert (a, 'A1');
+
+%!test
+%! a = calccelladdress (378, 28);
+%! assert (a, 'AB378');
+
+%!test
+%! a = calccelladdress (65536, 1024);
+%! assert (a, 'AMJ65536');
+
+%!test
+%! a = calccelladdress (1048576, 16384);
+%! assert (a, 'XFD1048576');
+
+%!test
+%! a = calccelladdress (378, 26);
+%! assert (a, 'Z378');
+
+%!test
+%! a = calccelladdress (378, 702);
+%! assert (a, 'ZZ378');
+
+%!test
+%! a = calccelladdress (378, 701);
+%! assert (a, 'ZY378');
+
+%!test
+%! a = calccelladdress (378, 703);
+%! assert (a, 'AAA378');
+
+%!test
+%! a = calccelladdress (378, 676);
+%! assert (a, 'YZ378');
+
+
