@@ -1,4 +1,4 @@
-## Copyright (C) 2009,2010 Philip Nienhuis <pr.nienhuis at users.sf.net>
+## Copyright (C) 2009,2010,2011,2012 Philip Nienhuis <pr.nienhuis at users.sf.net>
 ## 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -83,7 +83,7 @@
 ##   Put a character string into cell B15 in sheet Oldsheet3
 ## @end example
 ##
-## @seealso ods2oct, odsopen, odsclose, odsread, odswrite, odsfinfo
+## @seealso {ods2oct, odsopen, odsclose, odsread, odswrite, odsfinfo}
 ##
 ## @end deftypefn
 
@@ -102,18 +102,25 @@
 ## 2010-08-15 Texinfo header edits
 ## 2010-08-16 Added check on presence of output argument
 ## 2010-08-23 Added check on validity of ods file ptr
-##     "      Experimental support for odfdom 0.8.6 (in separate subfunc, to be integrated later)
+##    ''      Experimental support for odfdom 0.8.6 (in separate subfunc, to be integrated later)
 ## 2010-08-25 Improved help text (java memory, ranges)
 ## 2010-10-27 Improved file change tracking tru ods.changed
 ## 2010-11-12 Better input argument checks
 ## 2010-11-13 Reset ods.limits when read was successful
 ## 2010-11-13 Added check for 2-D input array
+## 2011-03-23 First try of odfdom 0.8.7
+## 2011-05-15 Experimental UNO support added
+## 2011-11-18 Fixed bug in test for range parameter being character string
+## 2012-01-26 Fixed "seealso" help string
+## 2012-02-20 Fixed range parameter to be default empty string rather than empty numeral
+## 2012-02-27 More range arg fixes
 ##
-## Last update of subfunctions below: 2012-12-08
+## Last update of subfunctions below: 2012-02-26
 
-function [ ods, rstatus ] = oct2ods (c_arr, ods, wsh=1, crange=[], spsh_opts=[])
+function [ ods, rstatus ] = oct2ods (c_arr, ods, wsh=1, crange='', spsh_opts=[])
 
 	if (nargin < 2) error ("oct2xls needs a minimum of 2 arguments."); endif
+  
 	# Check if input array is cell
 	if (isempty (c_arr))
 		warning ("Request to write empty matrix - ignored."); 
@@ -128,18 +135,26 @@ function [ ods, rstatus ] = oct2ods (c_arr, ods, wsh=1, crange=[], spsh_opts=[])
 		error ("oct2ods: input array neither cell nor numeric array");
 	endif
 	if (ndims (c_arr) > 2), error ("Only 2-dimensional arrays can be written to spreadsheet"); endif
-	# Check ods file pointer struct
+
+  # Check ods file pointer struct
 	test1 = ~isfield (ods, "xtype");
 	test1 = test1 || ~isfield (ods, "workbook");
 	test1 = test1 || isempty (ods.workbook);
 	test1 = test1 || isempty (ods.app);
 	if test1
-		error ("Invalid ods file pointer struct");
+		error ("Arg #2: Invalid ods file pointer struct");
 	endif
+
 	# Check worksheet ptr
 	if (~(ischar (wsh) || isnumeric (wsh))), error ("Integer (index) or text (wsh name) expected for arg # 3"); endif
+
 	# Check range
-	if (~(isempty (crange) || ischar (crange))), error ("Character string (range) expected for arg # 4"); endif
+	if (~isempty (crange) && ~ischar (crange))
+    error ("Character string (range) expected for arg # 4");
+  elseif (isempty (crange))
+    crange = '';
+  endif
+
 	# Various options 
 	if (isempty (spsh_opts))
 		spsh_opts.formulas_as_text = 0;
@@ -158,16 +173,23 @@ function [ ods, rstatus ] = oct2ods (c_arr, ods, wsh=1, crange=[], spsh_opts=[])
 		switch ods.odfvsn
 			case "0.7.5"
 				[ ods, rstatus ] = oct2jotk2ods (c_arr, ods, wsh, crange, spsh_opts);
-			case "0.8.6"
+			case {"0.8.6", "0.8.7"}
 				[ ods, rstatus ] = oct3jotk2ods (c_arr, ods, wsh, crange, spsh_opts);
 			otherwise
 				error ("Unsupported odfdom version");
 		endswitch
+
 	elseif (strcmp (ods.xtype, 'JOD'))
 		# Write ods file tru Java & jOpenDocument. API still leaves lots to be wished...
 		[ ods, rstatus ] = oct2jod2ods (c_arr, ods, wsh, crange);
+
+	elseif (strcmp (ods.xtype, 'UNO'))
+		# Write ods file tru Java & UNO bridge (OpenOffice.org & clones)
+		[ ods, rstatus ] = oct2uno2ods (c_arr, ods, wsh, crange, spsh_opts);
+
 #	elseif 
 		# ---- < Other interfaces here >
+
 	else
 		error (sprintf ("ods2oct: unknown OpenOffice.org .ods interface - %s.", ods.xtype));
 	endif
@@ -179,7 +201,7 @@ endfunction
 
 #=============================================================================
 
-## Copyright (C) 2010 Philip Nienhuis <prnienhuis@users.sf.net>
+## Copyright (C) 2010,2011 Philip Nienhuis <prnienhuis@users.sf.net>
 ## 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -621,7 +643,7 @@ endfunction
 
 #=============================================================================
 
-## Copyright (C) 2009 Philip Nienhuis <prnienhuis _at- users.sf.net>
+## Copyright (C) 2010,2011 Philip Nienhuis <prnienhuis _at- users.sf.net>
 ## 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -657,7 +679,8 @@ endfunction
 ## 2010-08-22 odfdom 0.8.6 is there... seems to work with just one bug, easily worked around
 ## 2010-10-27 Improved file change tracking tru ods.changed
 ## 2010-11-12 Improved file change tracking tru ods.changed
-#3 2010-12-08 Bugfixes (obj -> arg L.715; removed stray arg in call to spsh_prstype L.719)
+## 2010-12-08 Bugfixes (obj -> arg L.715; removed stray arg in call to spsh_prstype L.719)
+## 2011-03-23 First try of odfdom 0.8.7
 
 function [ ods, rstatus ] = oct3jotk2ods (c_arr, ods, wsh, crange, spsh_opts)
 
@@ -674,7 +697,11 @@ function [ ods, rstatus ] = oct3jotk2ods (c_arr, ods, wsh, crange, spsh_opts)
 								# makes physical copies only when needed (?)
 	odfroot = odfcont.getRootElement ();
 	offsprdsh = ods.app.getContentRoot();
-	spsh = odfcont.getOdfDocument ();
+	if (strcmp (ods.odfvsn, '0.8.7'))
+		spsh = odfcont.getDocument ();
+	else
+		spsh = odfcont.getOdfDocument ();
+	endif
 
 	# Get some basic spreadsheet data from the pointer using ODFtoolkit
 	autostyles = odfcont.getOrCreateAutomaticStyles();
@@ -814,7 +841,7 @@ endfunction
 
 #=============================================================================
 
-## Copyright (C) 2009-2010 Philip Nienhuis <pr.nienhuis at users.sf.net>
+## Copyright (C) 2009,2010,2011 Philip Nienhuis <pr.nienhuis at users.sf.net>
 ## 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -854,6 +881,7 @@ endfunction
 ## 2010-08-15 Fixed bug with invalid first sheet in new spreadsheets
 ## 2010-10-27 Improved file change tracking tru ods.changed
 ## 2010-11-12 Improved file change tracking tru ods.changed
+## 2012-02-26 Write logicals as doubles (bug in jOpenDocument, would write as text)
 
 function [ ods, rstatus ] = oct2jod2ods (c_arr, ods, wsh, crange)
 
@@ -952,6 +980,8 @@ function [ ods, rstatus ] = oct2jod2ods (c_arr, ods, wsh, crange)
 		for jj = 1 : ncols
 			val = c_arr {ii, jj};
 			if ((isnumeric (val) && ~isnan (val)) || ischar (val) || islogical (val))
+        # FIXME: jOpenDocument doesn't really support writing booleans (doesn't set OffValAttr)
+        if (islogical (val)); val = double (val); endif
 				try
 					sh.getCellAt (jj + lcol - 1, ii + trow - 1).clearValue();
 					jcell = sh.getCellAt (jj + lcol - 1, ii + trow - 1).setValue (val);
@@ -970,3 +1000,169 @@ function [ ods, rstatus ] = oct2jod2ods (c_arr, ods, wsh, crange)
 	endif
 
 endfunction
+
+
+## Copyright (C) 2011 Philip Nienhuis <prnienhuis@users.sf.net>
+## 
+## This program is free software; you can redistribute it and/or modify
+## it under the terms of the GNU General Public License as published by
+## the Free Software Foundation; either version 2 of the License, or
+## (at your option) any later version.
+## 
+## This program is distributed in the hope that it will be useful,
+## but WITHOUT ANY WARRANTY; without even the implied warranty of
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
+## 
+## You should have received a copy of the GNU General Public License
+## along with Octave; see the file COPYING.  If not, see
+## <http://www.gnu.org/licenses/>.
+
+## oct2uno2ods - transfer data to ods or xls file using Java/UNO bridge
+## with OpenOffice_org & clones
+
+## Author: Philip Nienhuis <prnienhuis@users.sf.net>
+## Created: 2011-05-15
+## Updates:
+## 2011-summer <many many improvements>
+## 2011-09-08 Stylistic changes
+## 2011-09-18 Adapted sh_names type to LO 3.4.1
+## 2011-09-23 Removed stray debug statements
+## 2012-02-25 Work around LO3.5rc1 Java Runtime Exception bug L.1043+
+## 2012-02-26 Bug fix when adding sheets near L.1101 (wrong if-else-end construct).
+
+function [ ods, rstatus ] = oct2uno2ods (c_arr, ods, wsh, crange, spsh_opts)
+
+  changed = 0;
+  newsh = 0;
+  ctype = [1, 2, 3, 4, 5];  # Float, Logical, String, Formula, Empty
+
+  # Get handle to sheet, create a new one if needed
+  sheets = ods.workbook.getSheets ();
+  sh_names = sheets.getElementNames ();
+  if (! iscell (sh_names))
+    # Java array (LibreOffice 3.4.+); convert to cellstr
+    sh_names = char (sh_names);
+  else
+    sh_names = {sh_names};
+  endif
+
+  # Clear default 2 last sheets in case of a new spreadsheet file
+  if (ods.changed > 2)
+    ii = numel (sh_names);
+    while (ii > 1)
+      shnm = sh_names{ii};
+      # Work around LibreOffice 3.5.rc1 bug (Java Runtime Exception)
+      try
+        sheets.removeByName (shnm);
+      end_try_catch
+      --ii;
+    endwhile
+    # Give remaining sheet a name
+    unotmp = java_new ('com.sun.star.uno.Type', 'com.sun.star.sheet.XSpreadsheet');
+    sh = sheets.getByName (sh_names{1}).getObject.queryInterface (unotmp);
+    if (isnumeric (wsh)); wsh = sprintf ("Sheet%d", wsh); endif
+    unotmp = java_new ('com.sun.star.uno.Type', 'com.sun.star.container.XNamed');
+    sh.queryInterface (unotmp).setName (wsh);
+  else
+
+    # Check sheet pointer
+    # FIXME sheet capacity check needed. How many can fit in an OOo sprsh.file?
+    if (isnumeric (wsh))
+      if (wsh < 1)
+        error ("Illegal sheet index: %d", wsh);
+      elseif (wsh > numel (sh_names))
+        # New sheet to be added. First create sheet name but check if it already exists
+        shname = sprintf ("Sheet%d", numel (sh_names) + 1);
+        jj = strmatch (wsh, sh_names);
+        if (~isempty (jj))
+          # New sheet name already in file, try to create a unique & reasonable one
+          ii = 1; filler = ''; maxtry = 5;
+          while (ii <= maxtry)
+            shname = sprintf ("Sheet%s%d", [filler "_"], numel (sh_names + 1));
+            if (isempty (strmatch (wsh, sh_names)))
+              ii = 10;
+            else
+              ++ii;
+            endif
+          endwhile
+          if (ii > maxtry + 1)
+            error ("Could not add sheet with a unique name to file %s");
+          endif
+        endif
+        wsh = shname;
+        newsh = 1;
+      else
+        # turn wsh index into the associated sheet name
+        wsh = sh_names (wsh);
+      endif
+    else
+      # wsh is a sheet name. See if it exists already
+      if (isempty (strmatch (wsh, sh_names)))
+        # Not found. New sheet to be added
+	      newsh = 1;
+      endif
+    endif
+    if (newsh)
+      # Add a new sheet. Sheet index MUST be a Java Short object
+      shptr = java_new ("java.lang.Short", sprintf ("%d", numel (sh_names) + 1));
+      sh = sheets.insertNewByName (wsh, shptr);
+    endif
+    # At this point we have a valid sheet name. Use it to get a sheet handle
+    unotmp = java_new ('com.sun.star.uno.Type', 'com.sun.star.sheet.XSpreadsheet');
+    sh = sheets.getByName (wsh).getObject.queryInterface (unotmp);
+  endif
+
+  # Check size of data array & range / capacity of worksheet & prepare vars
+  [nr, nc] = size (c_arr);
+  [topleft, nrows, ncols, trow, lcol] = spsh_chkrange (crange, nr, nc, ods.xtype, ods.filename);
+  --trow; --lcol;                      # Zero-based row # & col #
+  if (nrows < nr || ncols < nc)
+    warning ("Array truncated to fit in range");
+    c_arr = c_arr(1:nrows, 1:ncols);
+  endif
+	
+  # Parse data array, setup typarr and throw out NaNs  to speed up writing;
+  typearr = spsh_prstype (c_arr, nrows, ncols, ctype, spsh_opts, 0);
+  if ~(spsh_opts.formulas_as_text)
+    # Find formulas (designated by a string starting with "=" and ending in ")")
+    fptr = cellfun (@(x) ischar (x) && strncmp (x, "=", 1), c_arr);
+    typearr(fptr) = ctype(4);          # FORMULA
+  endif
+
+  # Transfer data to sheet
+  for ii=1:nrows
+    for jj=1:ncols
+      try
+        XCell = sh.getCellByPosition (lcol+jj-1, trow+ii-1);
+        switch typearr(ii, jj)
+          case 1	# Float
+            XCell.setValue (c_arr{ii, jj});
+          case 2	# Logical. Convert to float
+            XCell.setValue (double (c_arr{ii, jj}));
+          case 3	# String
+            unotmp = java_new ('com.sun.star.uno.Type', 'com.sun.star.text.XText');
+            XCell.queryInterface (unotmp).setString (c_arr{ii, jj});
+          case 4	# Formula
+            if (spsh_opts.formulas_as_text)
+              unotmp = java_new ('com.sun.star.uno.Type', 'com.sun.star.text.XText');
+              XCell.queryInterface (unotmp).setString (c_arr{ii, jj});
+            else
+              XCell.setFormula (c_arr{ii, jj});
+            endif
+          otherwise
+            # Empty cell
+        endswitch
+		    changed = 1;
+      catch
+        printf ("Error writing cell %s (typearr() = %d)\n", calccelladdress(trow+ii, lcol+jj), typearr(ii, jj));
+		  end_try_catch
+    endfor
+  endfor
+
+  if (changed)	
+    ods.changed = max (min (ods.changed, 2), changed);	# Preserve 2 (new file), 1 (existing)
+    rstatus = 1;
+  endif
+
+endfunction
