@@ -1,4 +1,4 @@
-## Copyright (C) 2009,2010 Philip Nienhuis <prnienhuis at users.sf.net>
+## Copyright (C) 2009,2010,2011,2012 Philip Nienhuis <prnienhuis at users.sf.net>
 ## 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -69,11 +69,12 @@
 ## Beware that -if invoked- Excel invocations may be left running silently
 ## in case of COM errors. Invoke xlsclose with proper pointer struct to
 ## close them.
-## When using java, note that large data array sizes elements may exhaust
-## the java shared memory space for the default java memory settings.
+## When using Java, note that large data array sizes elements may exhaust
+## the Java shared memory space for the default java memory settings.
 ## For larger arrays, appropriate memory settings are needed in the file
-## java.opts; then the maximum array size for the java-based spreadsheet
-## options may be in the order of 10^6 elements.
+## java.opts; then the maximum array size for the Java-based spreadsheet
+## options may be in the order of 10^6 elements. In caso of UNO this
+## limit is not applicable and spreadsheets may be much larger.
 ##
 ## Examples:
 ##
@@ -81,7 +82,7 @@
 ##   [xlso, status] = xls2oct ('arr', xlsi, 'Third_sheet', 'AA31:AB278');
 ## @end example
 ##
-## @seealso xls2oct, xlsopen, xlsclose, xlsread, xlswrite, xlsfinfo
+## @seealso {xls2oct, xlsopen, xlsclose, xlsread, xlswrite, xlsfinfo}
 ##
 ## @end deftypefn
 
@@ -101,10 +102,18 @@
 ## 2010-11-12 Moved ptr struct check into main func. More input validity checks
 ## 2010-11-13 Added check for 2-D input array
 ## 2010-12-01 Better check on file pointer struct (ischar (xls.xtype))
+## 2011-03-29 OpenXLS support added. Works but saving to file (xlsclose) doesn't work yet 
+##      ''    Bug fixes (stray variable c_arr, and wrong test for valid xls struct)
+## 2011-05-18 Experimental UNO support
+## 2011-09-08 Bug fix in range arg check; code cleanup
+## 2011-11-18 Fixed another bug in test for range parameter being character string
+## 2012-01-26 Fixed "seealso" help string
+## 2012-02-20 Fixed range parameter to be default empty string rather than empty numeral
+## 2012-02-27 More range param fixes
 
-## Last script file update (incl. subfunctions): 2011-11-12
+## Last script file update (incl. subfunctions): 2012-02-26
 
-function [ xls, rstatus ] = oct2xls (obj, xls, wsh=1, crange=[], spsh_opts=[])
+function [ xls, rstatus ] = oct2xls (obj, xls, wsh=1, crange='', spsh_opts=[])
 
 	if (nargin < 2) error ("oct2xls needs a minimum of 2 arguments."); endif
 	# Make sure input array is a cell array
@@ -120,21 +129,28 @@ function [ xls, rstatus ] = oct2xls (obj, xls, wsh=1, crange=[], spsh_opts=[])
 	elseif (~iscell (obj))
 		error ("oct2xls: input array neither cell nor numeric array");
 	endif
-	if (ndims (c_arr) > 2), error ("Only 2-dimensional arrays can be written to spreadsheet"); endif
-	
+	if (ndims (obj) > 2), error ("Only 2-dimensional arrays can be written to spreadsheet"); endif
+
 	# Check xls file pointer struct
 	test1 = ~isfield (xls, "xtype");
 	test1 = test1 || ~isfield (xls, "workbook");
 	test1 = test1 || isempty (xls.workbook);
 	test1 = test1 || isempty (xls.app);
-	test1 = test1 || ischar (xls.xtype);
+	test1 = test1 || ~ischar (xls.xtype);
 	if (test1)
 		error ("Invalid xls file pointer struct");
 	endif
+
 	# Check worksheet ptr
 	if (~(ischar (wsh) || isnumeric (wsh))), error ("Integer (index) or text (wsh name) expected for arg # 3"); endif
+
 	# Check range
-	if (~(isempty (crange) || ischar (crange))), error ("Character string (range) expected for arg # 4"); endif
+	if (~isempty (crange) && ~ischar (crange))
+    error ("Character string (range) expected for arg # 4");
+  elseif (isempty (crange))
+    crange = '';
+  endif
+
 	# Various options 
 	if (isempty (spsh_opts))
 		spsh_opts.formulas_as_text = 0;
@@ -149,16 +165,23 @@ function [ xls, rstatus ] = oct2xls (obj, xls, wsh=1, crange=[], spsh_opts=[])
 	if (nargout < 1) printf ("Warning: no output spreadsheet file pointer specified.\n"); endif
 	
 	# Select interface to be used
-	if (strcmp (xls.xtype, 'COM'))
+	if (strcmpi (xls.xtype, 'COM'))
 		# Call oct2com2xls to do the work
 		[xls, rstatus] = oct2com2xls (obj, xls, wsh, crange, spsh_opts);
-	elseif (strcmp (xls.xtype, 'POI'))
+	elseif (strcmpi (xls.xtype, 'POI'))
 		# Invoke Java and Apache POI
 		[xls, rstatus] = oct2jpoi2xls (obj, xls, wsh, crange, spsh_opts);
-	elseif (strcmp (xls.xtype, 'JXL'))
+	elseif (strcmpi (xls.xtype, 'JXL'))
 		# Invoke Java and JExcelAPI
 		[xls, rstatus] = oct2jxla2xls (obj, xls, wsh, crange, spsh_opts);
-#	elseif (strcmp'xls.xtype, '<whatever>'))
+	elseif (strcmpi (xls.xtype, 'OXS'))
+		# Invoke Java and OpenXLS     ##### Not complete, saving file doesn't work yet!
+		printf ('Sorry, writing with OpenXLS not reliable => not supported yet\n');
+#		[xls, rstatus] = oct2oxs2xls (obj, xls, wsh, crange, spsh_opts);
+	elseif (strcmpi (xls.xtype, 'UNO'))
+		# Invoke Java and UNO bridge (OpenOffice.org)
+		[xls, rstatus] = oct2uno2xls (obj, xls, wsh, crange, spsh_opts);
+#	elseif (strcmpi (xls.xtype, '<whatever>'))
 #		<Other Excel interfaces>
 	else
 		error (sprintf ("oct2xls: unknown Excel .xls interface - %s.", xls.xtype));
@@ -168,7 +191,7 @@ endfunction
 
 
 #===================================================================================
-## Copyright (C) 2009,2010 by Philip Nienhuis <prnienhuis@users.sf.net>
+## Copyright (C) 2009,2010,2011,2012 by Philip Nienhuis <prnienhuis@users.sf.net>
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -202,7 +225,7 @@ endfunction
 ##   xls = oct2com2xls (rand (10, 15), xls, 'Third_sheet', 'BF24');
 ## @end example
 ##
-## @seealso oct2xls, xls2oct, xlsopen, xlsclose, xlswrite, xlsread, xls2com2oct
+## @seealso {oct2xls, xls2oct, xlsopen, xlsclose, xlswrite, xlsread, xls2com2oct}
 ##
 ## @end deftypefn
 
@@ -221,18 +244,20 @@ endfunction
 ## 2010-08-01 Added range vs. array size vs. capacity checks
 ## 2010-08-03 Moved range checks and type array parsing to separate functions
 ## 2010-10-20 Bug fix removing new empty sheets in new workbook that haven't been 
-##            created in the first place duetoExcelsetting (thanks Ian Journeaux)
+##            created in the first place due to Excel setting (thanks Ian Journeaux)
 ##     "      Changed range use in COM transfer call
 ## 2010-10-21 Improved file change tracking (var xls.changed)
 ## 2010-10-24 Fixed bug introduced in above fix: for loops have no stride param,
 ##     "      replaced by while loop
 ##     "      Added check for "live" ActiveX server
 ## 2010-11-12 Moved ptr struct check into main func
+## 2012-01-26 Fixed "seealso" help string
+## 2012-02-27 Copyright strings updated
 
 function [ xls, status ] = oct2com2xls (obj, xls, wsh, crange, spsh_opts)
 
 	# Preliminary sanity checks
-	if (~strmatch (tolower (xls.filename(end-4:end)), '.xls'))
+	if (~strmatch (lower (xls.filename(end-4:end)), '.xls'))
 		error ("oct2com2xls can only write to Excel .xls or .xlsx files")
 	endif
 	if (isnumeric (wsh))
@@ -399,7 +424,7 @@ endfunction
 
 #====================================================================================
 
-## Copyright (C) 2009,2010 Philip Nienhuis <prnienhuis at users.sf.net>
+## Copyright (C) 2009,2010,2011,2012 Philip Nienhuis <prnienhuis at users.sf.net>
 ## 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -435,7 +460,7 @@ endfunction
 ##   [xlso, status] = xls2jpoi2oct ('arr', xlsi, 'Third_sheet', 'AA31');
 ## @end example
 ##
-## @seealso oct2xls, xls2oct, xlsopen, xlsclose, xlsread, xlswrite
+## @seealso {oct2xls, xls2oct, xlsopen, xlsclose, xlsread, xlswrite}
 ##
 ## @end deftypefn
 
@@ -454,6 +479,9 @@ endfunction
 ## 2010-10-21 Improved logic for tracking file changes
 ## 2010-10-27 File change tracking again refined, internal var 'changed' dropped
 ## 2010-11-12 Moved ptr struct check into main func
+## 2011-11-19 Try-catch added to allow for changed method name for nr of worksheets
+## 2012-01-26 Fixed "seealso" help string
+## 2012-02-27 Copyright strings updated
 
 function [ xls, rstatus ] = oct2jpoi2xls (obj, xls, wsh, crange, spsh_opts)
 
@@ -475,7 +503,11 @@ function [ xls, rstatus ] = oct2jpoi2xls (obj, xls, wsh, crange, spsh_opts)
 	rstatus = 0; f_errs = 0;
 
 	# Check if requested worksheet exists in the file & if so, get pointer
-	nr_of_sheets = xls.workbook.getNumberOfSheets ();
+	try
+    nr_of_sheets = xls.workbook.getNumWorkSheets ();
+  catch
+    nr_of_sheets = xls.workbook.getNumberOfSheets ();
+  end_try_catch
 	if (isnumeric (wsh))
 		if (wsh > nr_of_sheets)
 			# Watch out as a sheet called Sheet%d can exist with a lower index...
@@ -556,7 +588,7 @@ endfunction
 
 
 #====================================================================================
-## Copyright (C) 2009,2010 Philip Nienhuis <prnienhuis at users.sf.net>
+## Copyright (C) 2009,2010,2011,2012 Philip Nienhuis <prnienhuis at users.sf.net>
 ## 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -592,7 +624,7 @@ endfunction
 ##   [xlso, status] = oct2jxla2oct ('arr', xlsi, 'Third_sheet', 'AA31');
 ## @end example
 ##
-## @seealso oct2xls, xls2oct, xlsopen, xlsclose, xlsread, xlswrite, xls2jxla2oct
+## @seealso {oct2xls, xls2oct, xlsopen, xlsclose, xlsread, xlswrite, xls2jxla2oct}
 ##
 ## @end deftypefn
 
@@ -611,9 +643,11 @@ endfunction
 ## 2010-08-03 Moved range checks and cell type parsing to separate routines
 ## 2010-08-11 Moved addcell() into try-catch as it is addCell which throws fatal errors
 ## 2010-10-20 Improved logic for tracking file changes (xls.changed 2 or 3); dropped
-##     "      internal variable 'changed'
+##     ''      internal variable 'changed'
 ## 2010-10-27 File change tracking again refined
 ## 2010-11-12 Moved ptr struct check into main func
+## 2012-01-26 Fixed "seealso" help string
+## 2012-02-27 Copyright strings updated
 
 function [ xls, rstatus ] = oct2jxla2xls (obj, xls, wsh, crange, spsh_opts)
 
@@ -657,7 +691,7 @@ function [ xls, rstatus ] = oct2jxla2xls (obj, xls, wsh, crange, spsh_opts)
 			sh = wb.createSheet (strng, nr_of_sheets); ++nr_of_sheets;
 			xls.changed = min (xls.changed, 2);		# Keep a 2 in case of new file
 		else
-			sh = wb.getSheet (wsh - 1);				# POI sheet count 0-based
+			sh = wb.getSheet (wsh - 1);				# JXL sheet count 0-based
 		endif
 		shnames = char (wb.getSheetNames ());
 		printf ("(Writing to worksheet %s)\n", 	shnames {nr_of_sheets, 1});
@@ -707,7 +741,7 @@ function [ xls, rstatus ] = oct2jxla2xls (obj, xls, wsh, crange, spsh_opts)
 					# First make sure formula functions are all uppercase
 					obj{ii, jj} = toupper (obj{ii, jj});
 					# There's no guarantee for formula correctness, so....
-					try		# Actually JExcelAPI flags formula errors as warnings :-(
+					try		# Actually JExcelAPI flags formula errors as mere warnings :-(
 						tmp = java_new ('jxl.write.Formula', kk, ll, obj{ii, jj});
 						# ... while errors are actually detected in addCell(), so
 						#     that should be within the try-catch
@@ -734,3 +768,297 @@ function [ xls, rstatus ] = oct2jxla2xls (obj, xls, wsh, crange, spsh_opts)
 	rstatus = 1;
   
 endfunction
+
+
+## Copyright (C) 2011 Philip Nienhuis <prnienhuis@users.sf.net>
+## 
+## This program is free software; you can redistribute it and/or modify
+## it under the terms of the GNU General Public License as published by
+## the Free Software Foundation; either version 2 of the License, or
+## (at your option) any later version.
+## 
+## This program is distributed in the hope that it will be useful,
+## but WITHOUT ANY WARRANTY; without even the implied warranty of
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
+## 
+## You should have received a copy of the GNU General Public License
+## along with Octave; see the file COPYING.  If not, see
+## <http://www.gnu.org/licenses/>.
+
+## -*- texinfo -*-
+## @deftypefn {Function File} [ @var{xlso}, @var{rstatus} ] = oct2oxs2xls ( @var{arr}, @var{xlsi})
+## @deftypefnx {Function File} [ @var{xlso}, @var{rstatus} ] = oct2oxs2xls (@var{arr}, @var{xlsi}, @var{wsh})
+## @deftypefnx {Function File} [ @var{xlso}, @var{rstatus} ] = oct2oxs2xls (@var{arr}, @var{xlsi}, @var{wsh}, @var{range})
+## @deftypefnx {Function File} [ @var{xlso}, @var{rstatus} ] = oct2oxs2xls (@var{arr}, @var{xlsi}, @var{wsh}, @var{range}, @var{options})
+##
+## Add data in 1D/2D CELL array @var{arr} into spreadsheet cell range @var{range}
+## in worksheet @var{wsh} in an Excel spreadsheet file pointed to in structure
+## @var{range}.
+## Return argument @var{xlso} equals supplied argument @var{xlsi} and is
+## updated by oct2oxs2xls.
+##
+## oct2oxs2xls should not be invoked directly but rather through oct2xls.
+##
+## @end deftypefn
+
+## Author: Philip Nienhuis <prnienhuis@users.sf.net>
+## Created: 2011-03-29
+## Updates:
+##
+
+function [ xls, rstatus ] = oct2oxs2xls (obj, xls, wsh, crange, spsh_opts)
+
+	# Preliminary sanity checks
+	if (~strmatch (tolower (xls.filename(end-4:end-1)), '.xls'))	# No OOXML in OXS
+		error ("OXS can only write to Excel .xls files")
+	endif
+	
+	changed = 0;
+
+	persistent ctype;
+	if (isempty (ctype))
+		ctype = [1, 2, 3, 4, 5];
+		# Number, Boolean, String, Formula, Empty
+	endif
+	# scratch vars
+	rstatus = 0; f_errs = 0;
+	
+	# Prepare workbook pointer if needed
+	wb = xls.workbook;
+
+	# Check if requested worksheet exists in the file & if so, get pointer
+	nr_of_sheets = wb.getNumWorkSheets ();			# 1 based !!
+	if (isnumeric (wsh))
+		if (wsh > nr_of_sheets)
+			# Watch out as a sheet called Sheet%d can exist with a lower index...
+			strng = sprintf ("Sheet%d", wsh);
+			ii = 1;
+			try
+				# While loop should be inside try-catch
+				while (ii < 5)
+					sh = wb.getWorkSheet (strng)
+					strng = ['_' strng];
+					++ii;
+				endwhile
+			catch
+				# No worksheet named <strng> found => we can proceed
+			end_try_catch
+			if (ii >= 5) error (sprintf( " > 5 sheets named [_]Sheet%d already present!", wsh)); endif
+			sh = wb.createWorkSheet (strng); ++nr_of_sheets;
+			xls.changed = min (xls.changed, 2);		# Keep a 2 in case of new file
+		else
+			sh = wb.getWorkSheet (wsh - 1);			# OXS sheet count 0-based
+		endif
+		printf ("(Writing to worksheet %s)\n", sh.getSheetName ());
+	else
+		try
+			sh = wb.getWorkSheet (wsh);
+		catch
+			# Sheet not found, just create it
+			sh = wb.createWorkSheet (wsh); ++nr_of_sheets;
+			xls.changed = min (xls.changed, 2);		# Keep a 2 for new file
+		end_try_catch
+	endif
+
+	# Parse date ranges  
+	[nr, nc] = size (obj);
+	[topleft, nrows, ncols, trow, lcol] = spsh_chkrange (crange, nr, nc, xls.xtype, xls.filename);
+	if (nrows < nr || ncols < nc)
+		warning ("Array truncated to fit in range");
+		obj = obj(1:nrows, 1:ncols);
+	endif
+
+	# Prepare type array
+	typearr = spsh_prstype (obj, nrows, ncols, ctype, spsh_opts);
+	if ~(spsh_opts.formulas_as_text)
+		# Remove leading '=' from formula strings  //FIXME needs updating
+		fptr = ~(4 * (ones (size (typearr))) .- typearr);
+		obj(fptr) = cellfun (@(x) x(2:end), obj(fptr), "Uniformoutput", false); 
+	endif
+	clear fptr
+
+	for ii=1:ncols
+		for jj=1:nrows
+			try
+				# Set value
+				sh.getCell(jj+trow-2, ii+lcol-2).setVal (obj{jj, ii});  # Addr.cnt = 0-based
+				changed = 1;
+			catch
+				# Cell not existant. Add cell
+				if ~(typearr(jj, ii) == 5)
+					sh.add (obj{jj, ii}, jj+trow-2, ii+lcol-2);
+					changed = 1;
+				endif
+			end_try_catch
+		endfor
+	endfor
+
+	if (changed), xls.changed = max (xls.changed, 1); endif   # Preserve 2 for new files
+	rstatus = 1;
+
+endfunction
+
+
+## Copyright (C) 2011,2012 Philip Nienhuis <prnienhuis@users.sf.net>
+## 
+## This program is free software; you can redistribute it and/or modify
+## it under the terms of the GNU General Public License as published by
+## the Free Software Foundation; either version 2 of the License, or
+## (at your option) any later version.
+## 
+## This program is distributed in the hope that it will be useful,
+## but WITHOUT ANY WARRANTY; without even the implied warranty of
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
+## 
+## You should have received a copy of the GNU General Public License
+## along with Octave; see the file COPYING.  If not, see
+## <http://www.gnu.org/licenses/>.
+
+## oct2uno2xls
+
+## Author: Philip Nienhuis <prnienhuis@users.sf.net>
+## Created: 2011-05-18
+## 2011-09-18 Adapted sh_names type to LO 3.4.1
+## 2011-09-23 Removed stray debug statements
+## 2012-02-25 Fixed wrong var name in L.933
+## 2012-02-25 Catch stray Java RuntimeException when deleting sheets
+## 2012-02-26 Bug fix when adding sheets near L.994 (wrong if-else-end construct).
+## 2012-02-27 Copyright strings updated
+
+function [ xls, rstatus ] = oct2uno2xls (c_arr, xls, wsh, crange, spsh_opts)
+
+  changed = 0;
+  newsh = 0;
+  ctype = [1, 2, 3, 4, 5];  # Float, Logical, String, Formula, Empty
+
+  # Get handle to sheet, create a new one if needed
+  sheets = xls.workbook.getSheets ();
+  sh_names = sheets.getElementNames ();
+  if (! iscell (sh_names))
+    # Java array (LibreOffice 3.4.+); convert to cellstr
+    sh_names = char (sh_names);
+  else
+    sh_names = {sh_names};
+  endif
+
+  # Clear default 2 last sheets in case of a new spreadsheet file
+  if (xls.changed > 2)
+    ii = numel (sh_names);
+    while (ii > 1)
+      shnm = sh_names{ii};
+      try
+        # Catch harmless Java RuntimeException "out of range" in LibreOffice 3.5rc1
+        sheets.removeByName (shnm);
+      end_try_catch
+      --ii;
+    endwhile
+    # Give remaining sheet a name
+    unotmp = java_new ('com.sun.star.uno.Type', 'com.sun.star.sheet.XSpreadsheet');
+    sh = sheets.getByName (sh_names{1}).getObject.queryInterface (unotmp);
+    if (isnumeric (wsh)); wsh = sprintf ("Sheet%d", wsh); endif
+    unotmp = java_new ('com.sun.star.uno.Type', 'com.sun.star.container.XNamed');
+    sh.queryInterface (unotmp).setName (wsh);
+  else
+
+    # Check sheet pointer
+    # FIXME sheet capacity check needed
+    if (isnumeric (wsh))
+      if (wsh < 1)
+        error ("Illegal sheet index: %d", wsh);
+      elseif (wsh > numel (sh_names))
+        # New sheet to be added. First create sheet name but check if it already exists
+        shname = sprintf ("Sheet%d", numel (sh_names) + 1);
+        jj = strmatch (wsh, sh_names);
+        if (~isempty (jj))
+          # New sheet name already in file, try to create a unique & reasonable one
+          ii = 1; filler = ''; maxtry = 5;
+          while (ii <= maxtry)
+            shname = sprintf ("Sheet%s%d", [filler "_"], numel (sh_names + 1));
+            if (isempty (strmatch (wsh, sh_names)))
+              ii = 10;
+            else
+              ++ii;
+            endif
+          endwhile
+          if (ii > maxtry + 1)
+            error ("Could not add sheet with a unique name to file %s");
+          endif
+        endif
+        wsh = shname;
+        newsh = 1;
+      else
+        # turn wsh index into the associated sheet name
+        wsh = sh_names (wsh);
+      endif
+    else
+      # wsh is a sheet name. See if it exists already
+      if (isempty (strmatch (wsh, sh_names)))
+        # Not found. New sheet to be added
+        newsh = 1;
+      endif
+    endif
+    if (newsh)
+      # Add a new sheet. Sheet index MUST be a Java Short object
+      shptr = java_new ("java.lang.Short", sprintf ("%d", numel (sh_names) + 1));
+      sh = sheets.insertNewByName (wsh, shptr);
+    endif
+    # At this point we have a valid sheet name. Use it to get a sheet handle
+    unotmp = java_new ('com.sun.star.uno.Type', 'com.sun.star.sheet.XSpreadsheet');
+    sh = sheets.getByName (wsh).getObject.queryInterface (unotmp);
+  endif
+
+  # Check size of data array & range / capacity of worksheet & prepare vars
+  [nr, nc] = size (c_arr);
+  [topleft, nrows, ncols, trow, lcol] = spsh_chkrange (crange, nr, nc, xls.xtype, xls.filename);
+  --trow; --lcol;                      # Zero-based row # & col #
+  if (nrows < nr || ncols < nc)
+    warning ("Array truncated to fit in range");
+    c_arr = c_arr(1:nrows, 1:ncols);
+  endif
+	
+  # Parse data array, setup typarr and throw out NaNs  to speed up writing;
+  typearr = spsh_prstype (c_arr, nrows, ncols, ctype, spsh_opts, 0);
+  if ~(spsh_opts.formulas_as_text)
+    # Find formulas (designated by a string starting with "=" and ending in ")")
+    fptr = cellfun (@(x) ischar (x) && strncmp (x, "=", 1), c_arr);
+    typearr(fptr) = ctype(4);          # FORMULA
+  endif
+
+  # Transfer data to sheet
+  for ii=1:nrows
+    for jj=1:ncols
+      try
+        XCell = sh.getCellByPosition (lcol+jj-1, trow+ii-1);
+        switch typearr(ii, jj)
+          case 1	# Float
+            XCell.setValue (c_arr{ii, jj});
+          case 2	# Logical. Convert to float as OOo has no Boolean type
+            XCell.setValue (double (c_arr{ii, jj}));
+          case 3	# String
+            unotmp = java_new ('com.sun.star.uno.Type', 'com.sun.star.text.XText');
+            XCell.queryInterface (unotmp).setString (c_arr{ii, jj});
+          case 4	# Formula
+            if (spsh_opts.formulas_as_text)
+              unotmp = java_new ('com.sun.star.uno.Type', 'com.sun.star.text.XText');
+              XCell.queryInterface (unotmp).setString (c_arr{ii, jj});
+            else
+              XCell.setFormula (c_arr{ii, jj});
+            endif
+          otherwise
+            # Empty cell
+        endswitch
+		    changed = 1;
+      catch
+        printf ("Error writing cell %s (typearr() = %d)\n", calccelladdress(trow+ii, lcol+jj), typearr(ii, jj));
+		  end_try_catch
+    endfor
+  endfor
+
+  if (changed)	
+    xls.changed = max (min (xls.changed, 2), changed);	# Preserve 2 (new file), 1 (existing)
+    rstatus = 1;
+  endif
+
+endfunction
