@@ -1,4 +1,4 @@
-## Copyright (C) 2009,2010 Philip Nienhuis <pr.nienhuis at users.sf.net>
+## Copyright (C) 2009,2010,2011,2012 Philip Nienhuis <pr.nienhuis at users.sf.net>
 ## 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -21,10 +21,13 @@
 ## @deftypefnx {Function File} [ @var{rawarr}, @var{ods}, @var{rstatus} ] = ods2oct (@var{ods}, @var{wsh}, @var{range}, @var{options})
 ##
 ## Read data contained within range @var{range} from worksheet @var{wsh}
-## in an OpenOffice_org spreadsheet file pointed to in struct @var{ods}.
+## in an OpenOffice_org Calc spreadsheet file pointed to in struct @var{ods}.
+##
+## @var{ods} is supposed to have been created earlier by odsopen in the
+## same octave session.
 ##
 ## @var{wsh} is either numerical or text, in the latter case it is 
-## case-sensitive and it may be max. 31 characters long.
+## case-sensitive.
 ## Note that in case of a numerical @var{wsh} this number refers to the
 ## position in the worksheet stack, counted from the left in a Calc
 ## window. The default is numerical 1, i.e. the leftmost worksheet
@@ -36,30 +39,40 @@
 ## determined behind the scenes first; this can take some time.
 ##
 ## Optional argument @var{options}, a structure, can be used to
-## specify various read modes. Currently the only option field is
-## "formulas_as_text"; if set to TRUE or 1, spreadsheet formulas
-## (if at all present) are read as formula strings rather than the
-## evaluated formula result values. This only works for the OTK (Open
-## Document Toolkit) interface.
+## specify various read modes by setting option fields in the struct
+## to true (1) or false (0). Currently recognized option fields are:
 ##
-## If only the first argument is specified, ods2oct will try to read
-## all contents from the first = leftmost (or the only) worksheet (as
-## if a range of @'' (empty string) was specified).
+## @table @asis
+## @item "formulas_as_text"
+## If set to TRUE or 1, spreadsheet formulas (if at all present)
+## are read as formula strings rather than the evaluated formula
+## result values. This only works for the OTK and UNO interfaces.
+## The default value is 0 (FALSE).
+##
+## @item 'strip_array'
+## Set the value of this field set to TRUE or 1 to strip the returned
+## output array @var{rawarr} from empty outer columns and rows. The
+## spreadsheet cell rectangle limits from where the data actually
+## came will be updated. The default value is FALSE or 0 (no cropping).
+## @end table
+##
+## If only the first argument @var{ods} is specified, ods2oct will
+## try to read all contents from the first = leftmost (or the only)
+## worksheet (as if a range of @'' (empty string) was specified).
 ## 
 ## If only two arguments are specified, ods2oct assumes the second
 ## argument to be @var{wsh}. In that case ods2oct will try to read
 ## all data contained in that worksheet.
 ##
 ## Return argument @var{rawarr} contains the raw spreadsheet cell data.
+## Use parsecell() to separate numeric and text values from @var{rawarr}.
+##
 ## Optional return argument @var{ods} contains the pointer struct. Field
 ## @var{ods}.limits contains the outermost column and row numbers of the
 ## actually read cell range.
-## @var{rstatus} will be set to 1 if the requested data have been read
-## successfully, 0 otherwise.
-## Use parsecell() to separate numeric and text values from @var{rawarr}.
 ##
-## @var{ods} is supposed to have been created earlier by odsopen in the
-## same octave session. It is only referred to, not changed.
+## Optional return argument @var{rstatus} will be set to 1 if the
+## requested data have been read successfully, 0 otherwise.
 ##
 ## Erroneous data and empty cells turn up empty in @var{rawarr}.
 ## Date/time values in OpenOffice.org are returned as numerical values
@@ -67,17 +80,10 @@
 ## rewritten by OpenOffice.org into .ods format may have numerical date
 ## cells with base 01-01-1900 (same as MS-Excel).
 ##
-## Be aware that ods2oct trims @var{rawarr} from empty outer rows & columns, 
-## so any returned cell array may turn out to be smaller than requested
-## in @var{range}.
-##
 ## When reading from merged cells, all array elements NOT corresponding 
 ## to the leftmost or upper OpenOffice.org cell will be treated as if the
 ## "corresponding" cells are empty.
 ##
-## ods2oct is a mere wrapper for interface-dependent scripts (e.g.,
-## ods2jotk2oct and ods2jod2oct) that do the actual reading.
-##
 ## Examples:
 ##
 ## @example
@@ -91,7 +97,7 @@
 ##   [An, ods2, status] = ods2oct (ods2, 'Third_sheet');
 ## @end example
 ##
-## @seealso odsopen, odsclose, parsecell, odsread, odsfinfo, oct2ods, odswrite
+## @seealso {odsopen, odsclose, parsecell, odsread, odsfinfo, oct2ods, odswrite}
 ##
 ## @end deftypefn
 
@@ -109,11 +115,15 @@
 ## 2010-08-12 Added explicit support for jOpenDocument v 1.2b3+
 ## 2010-08-25 Improved helptext (moved some text around)
 ## 2010-08-27 Added ods3jotk2oct - internal function for odfdom-0.8.6.jar
-##      "     Extended check on spsh_opts (must be a struct) 
+##     ''     Extended check on spsh_opts (must be a struct) 
 ## 2010-10-27 Moved cropping rawarr from empty outer rows & columns to here
-## 2010-12-06 Textual changes to info header 
+## 2011-05-06 Experimental UNO support
+## 2011-09-18 Set rstatus var here
+## 2012-01-26 Fixed "seealso" help string
+## 2012-02-25 Added 0.8.7 to supported odfdom versions in L.155
+## 2012-02-26 Updated texinfo header help text
 ##
-## (Latest update of subfunctions below: 2010-11-13)
+## (Latest update of subfunctions below: 2012-02-25)
 
 function [ rawarr, ods, rstatus ] = ods2oct (ods, wsh=1, datrange=[], spsh_opts=[])
 
@@ -148,23 +158,28 @@ function [ rawarr, ods, rstatus ] = ods2oct (ods, wsh=1, datrange=[], spsh_opts=
 		# Read ods file tru Java & ODF toolkit
 		switch ods.odfvsn
 			case '0.7.5'
-				[rawarr, ods, rstatus] = ods2jotk2oct (ods, wsh, datrange, spsh_opts);
-			case '0.8.6'
-				[rawarr, ods, rstatus] = ods3jotk2oct (ods, wsh, datrange, spsh_opts);
+				[rawarr, ods] = ods2jotk2oct (ods, wsh, datrange, spsh_opts);
+			case {'0.8.6', '0.8.7'}
+				[rawarr, ods] = ods3jotk2oct (ods, wsh, datrange, spsh_opts);
 			otherwise
 				error ("Unsupported odfdom version or invalid ods file pointer.");
 		endswitch
 	elseif (strcmp (ods.xtype, 'JOD'))
 		# Read ods file tru Java & jOpenDocument. JOD doesn't know about formulas :-(
-		[rawarr, ods, rstatus] = ods2jod2oct  (ods, wsh, datrange);
+		[rawarr, ods] = ods2jod2oct  (ods, wsh, datrange);
+	elseif (strcmp (ods.xtype, 'UNO'))
+		# Read ods file tru Java & UNO
+		[rawarr, ods] = ods2uno2oct (ods, wsh, datrange, spsh_opts);
 #	elseif 
 	#	---- < Other interfaces here >
 	else
 		error (sprintf ("ods2oct: unknown OpenOffice.org .ods interface - %s.", ods.xtype));
 	endif
 
+  rstatus = ~isempty (rawarr);
+
 	# Optionally strip empty outer rows and columns & keep track of original data location
-	if (spsh_opts.strip_array)
+	if (spsh_opts.strip_array && rstatus)
 		emptr = cellfun ('isempty', rawarr);
 		if (all (all (emptr)))
 			rawarr = {};
@@ -191,7 +206,7 @@ endfunction
 
 #=====================================================================
 
-## Copyright (C) 2009,2010 Philip Nienhuis <prnienhuis _at- users.sf.net>
+## Copyright (C) 2009,2010,2011 Philip Nienhuis <prnienhuis _at- users.sf.net>
 ## 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -222,14 +237,13 @@ endfunction
 ## 2010-03-19 More code cleanup & fixes for bugs introduced 18/3/2010 8-()
 ## 2010-08-03 Added preliminary support for reading back formulas as text strings
 ## 2010-10-27 Moved cropping rawarr from empty outer rows & columns to caller
+## 2011-09-18 Remove rstatus var (now set in caller)
 
-function [ rawarr, ods, rstatus ] = ods2jotk2oct (ods, wsh, crange, spsh_opts)
+function [ rawarr, ods ] = ods2jotk2oct (ods, wsh, crange, spsh_opts)
 
 	# Parts after user gfterry in
 	# http://www.oooforum.org/forum/viewtopic.phtml?t=69060
 	
-	rstatus = 0;
-
 	# Get contents and table stuff from the workbook
 	odfcont = ods.workbook;		# Use a local copy just to be sure. octave 
 								# makes physical copies only when needed (?)
@@ -407,7 +421,7 @@ endfunction
 
 #===========================================================================
 
-## Copyright (C) 2010 Philip Nienhuis <prnienhuis@users.sf.net>
+## Copyright (C) 2010,2011 Philip Nienhuis <prnienhuis@users.sf.net>
 ## 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -430,10 +444,11 @@ endfunction
 ## Updates:
 ## 2010-10-27 Moved cropping rawarr from empty outer rows & columns to caller
 ## 2010-11-13 Added workaround for reading text cells in files made by jOpenDocument 1.2bx
+## 2011-09-18 Comment out workaround for jOpenDocument bug (no OfficeValueAttr set)
+##            because this casts all numeric cells to string type for properly written ODS1.2
+##     ''     Remove rstatus var (now set in caller)
 
-function [ rawarr, ods, rstatus ] = ods3jotk2oct (ods, wsh, crange, spsh_opts)
-
-	rstatus = 0;
+function [ rawarr, ods ] = ods3jotk2oct (ods, wsh, crange, spsh_opts)
 
 	# Get contents and table stuff from the workbook
 	odfcont = ods.workbook;		# Use a local copy just to be sure. octave 
@@ -499,16 +514,16 @@ function [ rawarr, ods, rstatus ] = ods3jotk2oct (ods, wsh, crange, spsh_opts)
 			ocell = row.getCellByIndex (jj-1);
 			if ~isempty (ocell)
 				otype = deblank (tolower (ocell.getValueType ()));
-				if (spsh_opts.formulas_as_text)
+   			if (spsh_opts.formulas_as_text)
 					if ~isempty (ocell.getFormula ())
 						otype = 'formula';
 					endif
 				endif
-				# Provisions for catching jOpenDocument 1.2b bug where text cells
-				# haven't been assigned an <office:value-type='string'> attribute
-				if (~isempty (ocell))
-					if (findstr ('<text:', char (ocell.getOdfElement ()))), otype = 'string'; endif
-				endif
+#				# Provisions for catching jOpenDocument 1.2b bug where text cells
+#				# haven't been assigned an <office:value-type='string'> attribute
+#				if (~isempty (ocell))
+#					if (findstr ('<text:', char (ocell.getOdfElement ()))), otype = 'string'; endif
+#				endif
 				# At last, read the data
 				switch otype
 					case  {'float', 'currency', 'percentage'}
@@ -541,7 +556,7 @@ function [ rawarr, ods, rstatus ] = ods3jotk2oct (ods, wsh, crange, spsh_opts)
 						rawarr(ii-trow+1, jj-lcol+1) = ocell.getBooleanValue ();
 					case 'string'
 						rawarr(ii-trow+1, jj-lcol+1) = ocell.getStringValue ();
-#						# Code left in for in case odfdom 0.8.6 has similar bug
+#						# Code left in for in case odfdom 0.8.6+ has similar bug
 #						# as 0.7.5
 #						cvalue = tcell.getOfficeStringValueAttribute ();
 #						if (isempty (cvalue))     # Happens with e.g., hyperlinks
@@ -574,7 +589,7 @@ endfunction
 
 #===========================================================================
 
-## Copyright (C) 2009,2010 Philip Nienhuis <pr.nienhuis at users.sf.net>
+## Copyright (C) 2009,2010,2011 Philip Nienhuis <pr.nienhuis at users.sf.net>
 ## 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -601,8 +616,14 @@ endfunction
 ##            allows better cell type parsing and is therefore more reliable
 ## 2010-10-27 Moved cropping rawarr from empty outer rows & columns to here
 ## 2010-11-13 Added workaround for reading text cells in files made by jOpenDocument 1.2bx
+## 2011-09-18 Comment out workaround for jOpenDocument bug (no OfficeValueAttr set)
+##            because this casts all numeric cells to string type for properly written ODS1.2
+##     ''     Remove rstatus var (now set in caller)
+## 2012-02-25 Fix reading string values written by JOD itself (no text attribue!!). But
+##            the cntents could be BOOLEAN as well (JOD doesn't write OffVal attr either)
+## 2012-02-26 Further workaround for reading strings (actually: cells w/o OfficeValueAttr)
 
-function [ rawarr, ods, rstatus] = ods2jod2oct (ods, wsh, crange)
+function [ rawarr, ods] = ods2jod2oct (ods, wsh, crange)
 
 	persistent months;
 	months = {"JAN", "FEB", "MAR", "APR", "MAY", "JUN", "JUL", "AUG", "SEP", "OCT", "NOV", "DEC"};
@@ -671,10 +692,6 @@ function [ rawarr, ods, rstatus] = ods2jod2oct (ods, wsh, crange)
 				try
 					scell = sh.getCellAt (lcol+jj-2, trow+ii-2);
 					sctype = char (scell.getValueType ());
-					# Workaround for sheets written by jOpenDocument 1.2bx (no value-type attrb):
-					if (~isempty (scell))
-						if (findstr ('<text:', char (scell))), sctype = STRING; endif
-					endif
 					switch sctype
 						case { FLOAT, CURRENCY, PERCENTAGE }
 							rawarr{ii, jj} = scell.getValue ().doubleValue ();
@@ -698,6 +715,12 @@ function [ rawarr, ods, rstatus] = ods2jod2oct (ods, wsh, crange)
 							ss = str2num (tmp{4}(7:8)) / 86600.0;
 							rawarr {ii, jj} = hh + mi + ss;
 						otherwise
+              # Workaround for sheets written by jOpenDocument (no value-type attrb):
+              if (~isempty (scell.getValue) )
+                # FIXME Assume cell contains string if there's a text attr. But it could be BOOLEAN too...
+                if (findstr ('<text:', char (scell))), sctype = STRING; endif
+                rawarr{ii, jj} = scell.getValue();
+              endif
 							# Nothing
 					endswitch
 				catch
@@ -752,7 +775,128 @@ function [ rawarr, ods, rstatus] = ods2jod2oct (ods, wsh, crange)
 
 	# Keep track of data rectangle limits
 	ods.limits = [lcol, rcol; trow, brow];
-	
-	rstatus = ~isempty (rawarr);
+
+endfunction
+
+
+## Copyright (C) 2011 Philip Nienhuis <prnienhuis@users.sf.net>
+## 
+## This program is free software; you can redistribute it and/or modify
+## it under the terms of the GNU General Public License as published by
+## the Free Software Foundation; either version 2 of the License, or
+## (at your option) any later version.
+## 
+## This program is distributed in the hope that it will be useful,
+## but WITHOUT ANY WARRANTY; without even the implied warranty of
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
+## 
+## You should have received a copy of the GNU General Public License
+## along with Octave; see the file COPYING.  If not, see
+## <http://www.gnu.org/licenses/>.
+
+## ods2uno2oct
+
+## Author: Philip Nienhuis <prnienhuis@users.sf.net>
+## Created: 2011-05-05
+## Updates:
+## 2011-09-18 Adapted sh_names type to LO 3.4.1
+##     ''     Remove default 2 last sheets (LibreOffice 3.4.+)
+##     ''     Remove rstatus var (now set in caller)
+## 2011-09-19 Try to decipher if formulas return numeric or string values
+
+function [rawarr, ods] = ods2uno2oct  (ods, wsh, datrange, spsh_opts)
+
+  sheets = ods.workbook.getSheets ();
+  sh_names = sheets.getElementNames ();
+  if (! iscell (sh_names))
+    # Java array (LibreOffice 3.4.+); convert to cellstr
+    sh_names = char (sh_names);
+  else
+    sh_names = {sh_names};
+  endif
+
+  # Check sheet pointer
+  if (isnumeric (wsh))
+	  if (wsh < 1 || wsh > numel (sh_names))
+      error ("Sheet index %d out of range 1-%d", wsh, numel (sh_names));
+    endif
+  else
+    ii = strmatch (wsh, sh_names);
+    if (isempty (ii)), error ("Sheet '%s' not found", wsh); endif
+    wsh = ii;
+  endif
+  unotmp = java_new ('com.sun.star.uno.Type', 'com.sun.star.sheet.XSpreadsheet');
+  sh = sheets.getByName(sh_names{wsh}).getObject.queryInterface (unotmp);
+
+  unotmp = java_new ('com.sun.star.uno.Type', 'com.sun.star.sheet.XCellRangesQuery');
+  xRQ = sh.queryInterface (unotmp);
+  # Get cell ranges of all rectangles containing data. Type values:
+  #java_get ('com.sun.star.sheet.CellFlags', 'VALUE')  ans =  1
+  #java_get ('com.sun.star.sheet.CellFlags', 'DATETIME')  ans =  2
+  #java_get ('com.sun.star.sheet.CellFlags', 'STRING')  ans =  4
+  #java_get ('com.sun.star.sheet.CellFlags', 'FORMULA')  ans =  16
+  # Yep, boolean is lacking...
+  Cellflgs = javaObject ("java.lang.Short", "23");
+  ccells = xRQ.queryContentCells (Cellflgs);
+  addrs = ccells.getRangeAddressesAsString ();
+
+  # Strip sheet name from addresses
+  adrblks = strsplit (addrs, ',');
+  if (isempty (adrblks))
+    warning ('Sheet %s contains no data', sh_names{wsh});
+    return
+  endif
+
+  # Either parse (given cell range) or prepare (unknown range) help variables.
+  # As OpenOffice knows the occupied range, we need the limits anyway to avoid
+  # out-of-range errors
+  [ trow, brow, lcol, rcol ] = getusedrange (ods, wsh);
+  if (isempty (datrange))
+    nrows = brow - trow + 1;	# Number of rows to be read
+    ncols = rcol - lcol + 1;	# Number of columns to be read
+  else
+    [dummy, nrows, ncols, srow, scol] = parse_sp_range (datrange);
+    # Truncate range silently if needed
+    brow = min (srow + nrows - 1, brow);
+    rcol = min (scol + ncols - 1, rcol);
+    trow = max (trow, srow);
+    lcol = max (lcol, scol);
+    nrows = min (brow - trow + 1, nrows);	# Number of rows to be read
+    ncols = min (rcol - lcol + 1, ncols);	# Number of columns to be read
+  endif
+  # Create storage for data at Octave side
+  rawarr = cell (nrows, ncols);
+
+  # Get data. Apparently row & column indices are 0-based in UNO
+  for ii=trow-1:brow-1
+    for jj=lcol-1:rcol-1
+      XCell = sh.getCellByPosition (jj, ii);
+      cType = XCell.getType().getValue ();
+      switch cType
+        case 1	# Value
+          rawarr{ii-trow+2, jj-lcol+2} = XCell.getValue ();
+        case 2	# String
+          unotmp = java_new ('com.sun.star.uno.Type', 'com.sun.star.text.XText');
+          rawarr{ii-trow+2, jj-lcol+2} = XCell.queryInterface (unotmp).getString ();
+        case 3	# Formula
+          if (spsh_opts.formulas_as_text)
+            rawarr{ii-trow+2, jj-lcol+2} = XCell.getFormula ();
+          else
+            # Unfortunately OOo gives no clue as to the type of formula result
+            unotmp = java_new ('com.sun.star.uno.Type', 'com.sun.star.text.XText');
+            rawarr{ii-trow+2, jj-lcol+2} = XCell.queryInterface (unotmp).getString ();
+            tmp = str2double (rawarr{ii-trow+2, jj-lcol+2});
+            # If the string happens to contain just a number we'll assume it is numeric
+            if (~isnan (tmp)); rawarr{ii-trow+2, jj-lcol+2} = tmp; endif
+          endif
+        otherwise
+          # Empty cell
+      endswitch
+    endfor
+  endfor 
+
+  # Keep track of data rectangle limits
+  ods.limits = [lcol, rcol; trow, brow];
 
 endfunction
