@@ -1,4 +1,4 @@
-## Copyright (C) 2009,2010 Philip Nienhuis <prnienhuis at users.sf.net>
+## Copyright (C) 2009,2010,2011,2012 Philip Nienhuis <prnienhuis at users.sf.net>
 ## 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -19,27 +19,34 @@
 ## @deftypefnx {Function File} @var{ods} = odsopen (@var{filename}, @var{readwrite})
 ## @deftypefnx {Function File} @var{ods} = odsopen (@var{filename}, @var{readwrite}, @var{reqintf})
 ## Get a pointer to an OpenOffice_org spreadsheet in the form of return
-## argument (file pointer struct) @var{ods}.
+## argument @var{ods}.
 ##
 ## Calling odsopen without specifying a return argument is fairly useless!
 ##
-## To make this function work at all, you need the Java package > 1.2.5 plus
-## ODFtoolkit (version 0.7.5 or 0.8.6) & xercesImpl, and/or jOpenDocument
-## installed on your computer + proper javaclasspath set. These interfaces are
-## referred to as OTK and JOD, resp., and are preferred in that order by default
-## (depending on their presence).
+## To make this function work at all, you need the Java package >= 1.2.5 plus
+## ODFtoolkit (version 0.7.5 or 0.8.6) & xercesImpl, and/or jOpenDocument, and/or
+## OpenOffice.org (or clones) installed on your computer + proper javaclasspath
+## set. These interfaces are referred to as OTK, JOD, and UNO resp., and are
+## preferred in that order by default (depending on their presence).
+## For (currently experimental) UNO support, Octave-Java package 1.2.8 + latest
+## fixes is imperative; furthermore the relevant classes had best be added to
+## the javaclasspath by utility function chk_spreadsheet_support().
 ##
 ## @var{filename} must be a valid .ods OpenOffice.org file name including
 ## .ods suffix. If @var{filename} does not contain any directory path,
 ## the file is saved in the current directory.
+## For UNO bridge, filenames need to be in the form "file:///<path_to_file>/filename";
+## a URL will also work. If a plain file name is given (absolute or relative),
+## odsopen() will transform it into proper form.
 ##
-## @var{readwrite} must be set to true ornumerical 1 if writing to spreadsheet
+## @var{readwrite} must be set to true or numerical 1 if writing to spreadsheet
 ## is desired immediately after calling odsopen(). It merely serves proper
 ## handling of file errors (e.g., "file not found" or "new file created").
 ##
 ## Optional input argument @var{reqintf} can be used to override the ODS
 ## interface automatically selected by odsopen. Currently implemented interfaces
-## are 'OTK' (Java / ODFtoolkit) or 'JOD' (Java / jOpenDocument).
+## are 'OTK' (Java/ODF Toolkit), 'JOD' (Java/jOpenDocument) and 'UNO'
+## (Java/OpenOffice.org UNO bridge).
 ##
 ## Examples:
 ##
@@ -52,7 +59,7 @@
 ##    the jOpenDocument interface is requested)
 ## @end example
 ##
-## @seealso odsclose, odsread, ods2oct, odsfinfo
+## @seealso {odsclose, odsread, ods2oct, odsfinfo, chk_spreadsheet_support}
 ##
 ## @end deftypefn
 
@@ -76,49 +83,71 @@
 ## 2010-10-27 Improved tracking of file changes tru ods.changed
 ## 2010-11-12 Small changes to help text
 ##     "      Added try-catch to file open sections to create fallback to other intf
-## 2010-12-06 Textual changes to info header 
+## 2011-05-06 Experimental UNO support
+## 2011-05-18 Creating new spreadsheet docs in UNO now works
+## 2011-06-06 Tamed down interface verbosity on first startup
+##      "     Multiple requested interfaces now possible 
+## 2011-09-03 Reset chkintf if no ods support was found to allow full interface rediscovery
+##            (otherwise javaclasspath additions will never be picked up)
+## 2012-01-26 Fixed "seealso" help string
+## 2012-02-26 Added ";" to suppress echo of filename f UNO
 ##
-## Latest change on subfunction below: 2011-02-15
+## Latest change on subfunctions below: 2011-09-18
 
 function [ ods ] = odsopen (filename, rw=0, reqinterface=[])
 
 	persistent odsinterfaces; persistent chkintf;
 	if (isempty (chkintf))
-		odsinterfaces = struct ( "OTK", [], "JOD", [] );
+		odsinterfaces = struct ( "OTK", [], "JOD", [], "UNO", [] );
 		chkintf = 1;
 	endif
 	
 	if (nargout < 1) usage ("ODS = odsopen (ODSfile, [Rw]). But no return argument specified!"); endif
 
-	## FIXME: if ever another interface is implemented the below stanzas
-	## should be remodeled after xlsopen() to allow for multiple
-	## user-desired interface requests (for just 2 interfaces there's
-	## no need yet)
 	if (~isempty (reqinterface))
-		# Try to invoke requested interface for this call. Check if it
-		# is supported anyway by emptying the corresponding var.
-		if (strcmp (tolower (reqinterface), tolower ('OTK')))
-			printf ("Java/ODFtoolkit interface requested... ");
-			odsinterfaces.OTK = []; odsinterfaces.JOD = 0;
-		elseif (strcmp (tolower (reqinterface), tolower ('JOD')))
-			printf ("Java/jOpenDocument interface requested... "); 
-			odsinterfaces.OTK = 0; odsinterfaces.JOD = [];
-		else
-			usage (sprintf ("Unknown .ods interface \"%s\" requested. Only OTK or JOD supported", reqinterface));
-		endif
-		[odsinterfaces] = getodsinterfaces (odsinterfaces);
-
-		# Well, is the requested interface supported on the system?
-		if (~odsinterfaces.(toupper (reqinterface)))
-			# No it aint
-			error (" ...but %s is not supported.", toupper (reqinterface));
+		if ~(ischar (reqinterface) || iscell (reqinterface)), usage ("Arg # 3 not recognized"); endif
+		# Turn arg3 into cell array if needed
+		if (~iscell (reqinterface)), reqinterface = {reqinterface}; endif
+		odsinterfaces.OTK = 0; odsinterfaces.JOD = 0; odsinterfaces.UNO = 0;
+		for ii=1:numel (reqinterface)
+			reqintf = toupper (reqinterface {ii});
+			# Try to invoke requested interface(s) for this call. Check if it
+			# is supported anyway by emptying the corresponding var.
+			if     (strcmp (reqintf, 'OTK'))
+				odsinterfaces.OTK = [];
+			elseif (strcmp (reqintf, 'JOD'))
+				odsinterfaces.JOD = [];
+			elseif (strcmp (reqintf, 'UNO'))
+				odsinterfaces.UNO = [];
+			else 
+				usage (sprintf ("Unknown .ods interface \"%s\" requested. Only OTK, JOD or UNO supported\n", reqinterface{}));
+			endif
+		endfor
+		printf ("Checking requested interface(s):\n");
+		odsinterfaces = getodsinterfaces (odsinterfaces);
+		# Well, is/are the requested interface(s) supported on the system?
+		# FIXME check for multiple interfaces
+		odsintf_cnt = 0;
+		for ii=1:numel (reqinterface)
+			if (~odsinterfaces.(toupper (reqinterface{ii})))
+				# No it aint
+				printf ("%s is not supported.\n", toupper (reqinterface{ii}));
+			else
+				++odsintf_cnt;
+			endif
+		endfor
+		# Reset interface check indicator if no requested support found
+		if (~odsintf_cnt)
+			chkintf = [];
+			ods = [];
+			return
 		endif
 	endif
 	
 	# Var rw is really used to avoid creating files when wanting to read, or
 	# not finding not-yet-existing files when wanting to write.
 
-	if (rw) rw = 1; endif		# Be sure it's either 0 or 1 initially
+	if (rw), rw = 1; endif		# Be sure it's either 0 or 1 initially
 
 	# Check if ODS file exists. Set open mode based on rw argument
 	if (rw), fmode = 'r+b'; else fmode = 'rb'; endif
@@ -215,7 +244,7 @@ function [ ods ] = odsopen (filename, rw=0, reqinterface=[])
 			catch
 				# 1.2b2 has not
 				ods.odfvsn = 2;
-				printf ("NOTE: jOpenDocument v. 1.2b2 has limited functionality. Try upgrading to 1.2b3+\n");
+				printf ("NOTE: jOpenDocument v. 1.2b2 has limited functionality. Try upgrading to 1.2\n");
 			end_try_catch
 			odssupport += 2;
 		catch
@@ -223,6 +252,56 @@ function [ ods ] = odsopen (filename, rw=0, reqinterface=[])
 		end_try_catch
 	endif
 
+	if (odsinterfaces.UNO && ~odssupport)
+		# First the file name must be transformed into a URL
+		if (~isempty (strmatch ("file:///", filename)) || ~isempty (strmatch ("http:///", filename))...
+			|| ~isempty (strmatch ("ftp:///", filename)) || ~isempty (strmatch ("www:///", filename)))
+			# Seems in proper shape for OOO (at first sight)
+		else
+			# Transform into URL form
+			fname = canonicalize_file_name (strsplit (filename, filesep){end});
+			# On Windows, change backslash file separator into forward slash
+			if (strcmp (filesep, "\\"))
+				tmp = strsplit (fname, filesep);
+				flen = numel (tmp);
+				tmp(2:2:2*flen) = tmp;
+				tmp(1:2:2*flen) = '/';
+				fname = [ tmp{:} ];
+			endif
+			filename = [ 'file://' fname ];
+		endif
+		try
+			xContext = java_invoke ("com.sun.star.comp.helper.Bootstrap", "bootstrap");
+			xMCF = xContext.getServiceManager ();
+			oDesktop = xMCF.createInstanceWithContext ("com.sun.star.frame.Desktop", xContext);
+			# Workaround:
+			unotmp = java_new ('com.sun.star.uno.Type', 'com.sun.star.frame.XComponentLoader');
+			aLoader = oDesktop.queryInterface (unotmp);
+			# Some trickery as Octave Java cannot create non-numeric arrays
+			lProps = javaArray ('com.sun.star.beans.PropertyValue', 1);
+			lProp = java_new ('com.sun.star.beans.PropertyValue', "Hidden", 0, true, []);
+			lProps(1) = lProp;
+			if (rw > 2)
+				xComp = aLoader.loadComponentFromURL ("private:factory/scalc", "_blank", 0, lProps);
+			else
+				xComp = aLoader.loadComponentFromURL (filename, "_blank", 0, lProps);
+			endif
+			# Workaround:
+			unotmp = java_new ('com.sun.star.uno.Type', 'com.sun.star.sheet.XSpreadsheetDocument');
+			# save in ods struct:
+			xSpdoc = xComp.queryInterface (unotmp);
+			ods.workbook = xSpdoc;			# Needed to be able to close soffice in odsclose()
+			ods.filename = filename;
+			ods.xtype = 'UNO';
+			ods.app.xComp = xComp;			# Needed to be able to close soffice in odsclose()
+			ods.app.aLoader = aLoader;	# Needed to be able to close soffice in odsclose()
+			ods.odfvsn = 'UNO';
+			odssupport += 4;
+		catch
+			error ('Couldn''t open file %s using UNO', filename);
+		end_try_catch
+	endif
+
 #	if 
 #		<other interfaces here>
 
@@ -241,7 +320,7 @@ function [ ods ] = odsopen (filename, rw=0, reqinterface=[])
 		if (ods.changed == 1) ods.changed = 0; endif
 	endif
 
-	if (~isempty (reqinterface))
+	if (~isempty (reqinterface) || ~odssupport)
 		# Reset all found interfaces for re-testing in the next call. Add interfaces if needed.
 		chkintf = [];
 	endif
@@ -249,7 +328,7 @@ function [ ods ] = odsopen (filename, rw=0, reqinterface=[])
 endfunction
 
 
-## Copyright (C) 2009,2010 Philip Nienhuis <prnienhuis at users.sf.net>
+## Copyright (C) 2009,2010,2011 Philip Nienhuis <prnienhuis at users.sf.net>
 ## 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -277,6 +356,7 @@ endfunction
 ## Currently implemented interfaces comprise:
 ## - Java & ODFtoolkit (www.apache.org)
 ## - Java & jOpenDocument (www.jopendocument.org)
+## - Java & UNO bridge (OpenOffice.org)
 ##
 ## Examples:
 ##
@@ -301,38 +381,65 @@ endfunction
 ##     "      Rearranged code a bit; fixed typos in OTK detection code (odfdvsn -> odfvsn)
 ## 2010-09-27 More code cleanup
 ## 2010-11-12 Warning added about waning support for odfdom v. 0.7.5
-## 2011-02-15 Adapted to new java-1.2.8 javaclasspath() calling style
+## 2011-05-06 Fixed wrong strfind tests
+##     "      Experimental UNO support added
+## 2011-05-18 Forgot to initialize odsinterfaces.UNO
+## 2011-06-06 Fix for javaclasspath format in *nix w. java-1.2.8 pkg
+##      "     Implemented more rigid Java check
+##      "     Tamed down verbosity
+## 2011-09-03 Fixed order of odsinterfaces.<member> statement in Java detection try-catch
+##      "     Reset tmp1 (always allow interface rediscovery) for empty odsinterfaces arg
+## 2011-09-18 Added temporary warning about UNO interface
 
 function [odsinterfaces] = getodsinterfaces (odsinterfaces)
 
-	if (isempty (odsinterfaces.OTK) && isempty (odsinterfaces.JOD))
-		chk1 = 1;
-		printf ("Supported interfaces: ");
-	else
-		chk1= 0;
-	endif
+	persistent tmp1 = []; persistent jcp;
+  persistent uno_1st_time = [];
 
+	if (isempty (odsinterfaces.OTK) && isempty (odsinterfaces.JOD) && isempty (odsinterfaces.UNO))
+    # Assume no interface detection has happened yet
+		printf ("Detected interfaces: ");
+    tmp1 = [];
+	elseif (isempty (odsinterfaces.OTK) || isempty (odsinterfaces.JOD) || isempty (odsinterfaces.UNO))
+    # At least one interface is requested by user
+		tmp1 = [];
+	endif
+	deflt = 0;
+
+	if (isempty (tmp1))
 	# Check Java support
-	try
-		tmp1 = javaclasspath ('-all');						# For java pkg > 1.2.7
-		if (isempty (tmp1)), tmp1 = javaclasspath; endif	# For java pkg < 1.2.8
-		# If we get here, at least Java works. Now check for proper entries
-		# in class path. Under *nix the classpath must first be split up
-		if (isunix) tmp1 = strsplit (char(tmp1), ":"); endif
-		## FIXME implement more rigid Java version check a la xlsopen.
-		## ods / Java stuff is less critical than xls / Java, however
-	catch
-		# No Java support
-		odsinterfaces.OTK = 0;
-		odsinterfaces.JOD = 0;
-		if ~(isempty (odsinterfaces.POI) && isempty (odsinterfaces.JXL))
-			# Some Java-based interface requested but Java support is absent
-			error ('No Java support found.');
-		else
-			# No specific Java-based interface requested. Just return
-			return;
-		endif
-	end_try_catch
+		try
+			jcp = javaclasspath ("-all");					# For java pkg >= 1.2.8
+			if (isempty (jcp)), jcp = javaclasspath; endif	# For java pkg <  1.2.8
+			# If we get here, at least Java works. Now check for proper version (>= 1.6)
+			jver = char (java_invoke ('java.lang.System', 'getProperty', 'java.version'));
+			cjver = strsplit (jver, '.');
+			if (sscanf (cjver{2}, '%d') < 6)
+				warning ("\nJava version too old - you need at least Java 6 (v. 1.6.x.x)\n");
+				return
+			endif
+			# Now check for proper entries in class path. Under *nix the classpath
+			# must first be split up. In java 1.2.8+ javaclasspath is already a cell array
+			if (isunix && ~iscell (jcp)), jcp = strsplit (char (jcp), pathsep ()); endif
+			tmp1 = 1;
+		catch
+			# No Java support
+			if ~(isempty (odsinterfaces.OTK) && isempty (odsinterfaces.JOD) && isempty (odsinterfaces.UNO))
+				# Some Java-based interface requested but Java support is absent
+				error ('No Java support found.');
+			else
+				# No specific Java-based interface requested. Just return (needed for if
+				# ever a non-Java based interfaces will be implemented)
+        odsinterfaces.OTK = 0;
+        odsinterfaces.JOD = 0;
+        odsinterfaces.UNO = 0;
+				return;
+			endif
+		end_try_catch
+	elseif (~tmp1)
+		% Earlier on no Java support detected
+		error (" No Java support found.");
+	endif
 
 	# Try Java & ODF toolkit
 	if (isempty (odsinterfaces.OTK))
@@ -340,12 +447,12 @@ function [odsinterfaces] = getodsinterfaces (odsinterfaces)
 		jpchk = 0; entries = {"odfdom", "xercesImpl"};
 		# Only under *nix we might use brute force: e.g., strfind(classpath, classname);
 		# under Windows we need the following more subtle, platform-independent approach:
-		for ii=1:length (tmp1)
+		for ii=1:length (jcp)
 			for jj=1:length (entries)
-				if (strfind ( tmp1{ii}, entries{jj})), ++jpchk; endif
+				if (~isempty (strfind ( jcp{ii}, entries{jj}))), ++jpchk; endif
 			endfor
 		endfor
-		if (jpchk >= 2)		# Apparently all requested classes present.
+		if (jpchk >= numel(entries))		# Apparently all requested classes present.
 			# Only now we can check for proper odfdom version (only 0.7.5 & 0.8.6 work OK).
 			# The odfdom team deemed it necessary to change the version call so we need this:
 			odfvsn = ' ';
@@ -355,15 +462,15 @@ function [odsinterfaces] = getodsinterfaces (odsinterfaces)
 			catch
 				odfvsn = java_invoke ('org.odftoolkit.odfdom.Version', 'getApplicationVersion');
 			end_try_catch
-			if ~(strcmp (odfvsn, '0.7.5') || strcmp (odfvsn, '0.8.6'))
-				warning ("\nodfdom version %s is not supported - use v. 0.7.5 or 0.8.6.\n", odfvsn);
+			if ~(strcmp (odfvsn, '0.7.5') || strcmp (odfvsn, '0.8.6') || strcmp (odfvsn, '0.8.7'))
+				warning ("\nodfdom version %s is not supported - use v. 0.8.6 or 0.8.7.\n", odfvsn);
 			else
 				if (strcmp (odfvsn, '0.7.5'))
 					warning ("odfdom v. 0.7.5 support won't be maintained - please upgrade to 0.8.6 or higher."); 
 				endif
 				odsinterfaces.OTK = 1;
-				printf (" Java/ODFtoolkit (OTK) OK. ");
-				chk1 = 1;
+				printf ("OTK");
+				if (deflt), printf ("; "); else, printf ("*; "); deflt = 1; endif
 			endif
 			odsinterfaces.odfvsn = odfvsn;
 		else
@@ -375,22 +482,56 @@ function [odsinterfaces] = getodsinterfaces (odsinterfaces)
 	if (isempty (odsinterfaces.JOD))
 		odsinterfaces.JOD = 0;
 		jpchk = 0; entries = {"jOpenDocument"};
-		for ii=1:length (tmp1)
+		for ii=1:length (jcp)
 			for jj=1:length (entries)
-				if (strfind (tmp1{ii}, entries{jj})), ++jpchk; endif
+				if (~isempty (strfind (jcp{ii}, entries{jj}))), ++jpchk; endif
 			endfor
 		endfor
-		if (jpchk >= 1)
+		if (jpchk >= numel(entries))
 			odsinterfaces.JOD = 1;
-			printf (" Java/jOpenDocument (JOD) OK. ");
-			chk1 = 1;
+			printf ("JOD");
+			if (deflt), printf ("; "); else, printf ("*; "); deflt = 1; endif
 		else
 			warning ("\nNot all required classes (.jar) in classpath for JOD");
 		endif
 	endif
+
+	# Try Java & UNO
+	if (isempty (odsinterfaces.UNO))
+		odsinterfaces.UNO = 0;
+		# entries(1) = not a jar but a directory (<000_install_dir/program/>)
+		jpchk = 0; entries = {'program', 'unoil', 'jurt', 'juh', 'unoloader', 'ridl'};
+		for jj=1:numel (entries)
+			for ii=1:numel (jcp)
+				jcplst = strsplit (jcp{ii}, filesep);
+				jcpentry = jcplst {end};
+				if (~isempty (strfind (lower (jcpentry), lower (entries{jj}))))
+					jpchk = jpchk + 1;
+				endif
+			endfor
+		endfor
+		if (jpchk >= numel (entries))
+			odsinterfaces.UNO = 1;
+			printf ("UNO");
+			if (deflt), printf ("; "); else, printf ("*; "); deflt = 1; endif
+		else
+			warning ("\nOne or more UNO classes (.jar) missing in javaclasspath");
+		endif
+	endif
 	
 	# ---- Other interfaces here, similar to the ones above
 
-	if (chk1) printf ("\n"); endif
+	if (deflt), printf ("(* = active interface)\n"); endif
+
+  ## FIXME the below stanza should be dropped once UNO is stable.
+  # Echo a suitable warning about experimental status:
+  if (isempty (uno_1st_time) && odsinterfaces.UNO);
+    uno_1st_time = 1;
+    printf ("\nPLEASE NOTE: UNO (=OpenOffice.org-behind-the-scenes) is EXPERIMENTAL\n");
+    printf ("After you've opened a spreadsheet file using the UNO interface,\n");
+    printf ("odsclose on that file will kill ALL OpenOffice.org invocations,\n");
+    printf ("also those that were started outside and/or before Octave!\n");
+    printf ("Trying to quit Octave w/o invoking odsclose will only hang Octave.\n\n");
+  endif
 	
 endfunction
