@@ -1,4 +1,4 @@
-## Copyright (C) 2009 Philip Nienhuis <prnienhuis at users.sf.net>
+## Copyright (C) 2009,2010,2011,2012 Philip Nienhuis <prnienhuis at users.sf.net>
 ## 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -49,7 +49,7 @@
 ##    text data into array Tn.)
 ## @end example
 ##
-## @seealso xlsread, odsread, xls2oct, ods2oct
+## @seealso {xlsread, odsread, xls2oct, ods2oct}
 ##
 ## @end deftypefn
 
@@ -59,6 +59,10 @@
 ## 2009-12-29
 ## 2010-08-25 Added option for second argument to be a file ptr
 ## 2010-10-15 Simplified code for numerical array
+## 2011-05-17 Fixed subscript indexing bug in cropping section when rawarr is
+##      "     numeric scalar
+## 2011-09-08 Copyright string updated
+## 2012-01-26 Fixed "seealso" help string
 
 function [ numarr, txtarr, lim ] = parsecell (rawarr, arg2=[])
 
@@ -146,7 +150,10 @@ function [ numarr, txtarr, lim ] = parsecell (rawarr, arg2=[])
 				rawarr = rawarr (irowt:irowb, icoll:icolr);
 				# Build numerical array
 				numarr = zeros (irowb-irowt+1, icolr-icoll+1);
-				numarr (emptr(irowt:irowb, icoll:icolr)) = NaN;
+				# Watch out for scalar (non-empty) numarr where emptr = 0
+				if (sum (emptr(:)) > 0)
+					numarr (emptr(irowt:irowb, icoll:icolr)) = NaN;
+				endif
 				numarr(~emptr(irowt:irowb, icoll:icolr)) = cell2mat (rawarr(~emptr(irowt:irowb, icoll:icolr)));
 				# Save limits
 				lim.numlimits = [icoll, icolr; irowt, irowb];
