@@ -1,4 +1,4 @@
-## Copyright (C) 2009,2010 Philip Nienhuis <prnienhuis at users.sf.net>
+## Copyright (C) 2009,2010,2011,12 Philip Nienhuis <prnienhuis at users.sf.net>
 ## 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -23,6 +23,9 @@
 ## Read data contained within range @var{range} from worksheet @var{wsh}
 ## in an Excel spreadsheet file pointed to in struct @var{xls}.
 ##
+## @var{xls} is supposed to have been created earlier by xlsopen in the
+## same octave session.
+##
 ## @var{wsh} is either numerical or text, in the latter case it is 
 ## case-sensitive and it may be max. 31 characters long.
 ## Note that in case of a numerical @var{wsh} this number refers to the
@@ -39,31 +42,41 @@
 ## reliable in this respect albeit much slower.
 ##
 ## Optional argument @var{options}, a structure, can be used to
-## specify various read modes. Currently the only option field is
-## "formulas_as_text"; if set to TRUE or 1, spreadsheet formulas
-## (if at all present) are read as formula strings rather than the
-## evaluated formula result values. This only works for the Java
-## based interfaces (POI and JXL).
+## specify various read modes by setting option fields in the struct
+## to true (1) or false (0). Currently recognized option fields are:
 ##
-## If only the first argument is specified, xls2oct will try to read
-## all contents from the first = leftmost (or the only) worksheet (as
-## if a range of @'' (empty string) was specified).
+## @table @asis
+## @item "formulas_as_text"
+## If set to TRUE or 1, spreadsheet formulas (if at all present)
+## are read as formula strings rather than the evaluated formula
+## result values. This only works for the Java based interfaces
+## POI, JXL and UNO. The default value is 0 (FALSE).
+##
+## @item 'strip_array'
+## Set the value of this field set to TRUE or 1 to strip the returned
+## output array @var{rawarr} from empty outer columns and rows. The
+## spreadsheet cell rectangle limits from where the data actually
+## came will be updated. The default value is FALSE or 0 (no cropping).
+## When using the COM interface, the output array is always cropped.
+## @end table
+##
+## If only the first argument @var{xls} is specified, xls2oct will try
+## to read all contents from the first = leftmost (or the only)
+## worksheet (as if a range of @'' (empty string) was specified).
 ## 
 ## If only two arguments are specified, xls2oct assumes the second
 ## argument to be @var{wsh}. In that case xls2oct will try to read
 ## all data contained in that worksheet.
 ##
 ## Return argument @var{rawarr} contains the raw spreadsheet cell data.
-## Optional return argument @var{xls} contains the pointer struct,
-## @var{rstatus} will be set to 1 if the requested data have been read
-## successfully, 0 otherwise.
 ## Use parsecell() to separate numeric and text values from @var{rawarr}.
 ##
-## @var{xls} is supposed to have been created earlier by xlsopen in the
-## same octave session. It is only referred to, not changed.
+## Optional return argument @var{xls} contains the pointer struct,
+## If any data have been read, field @var{xls}.limits contains the
+## outermost column and row numbers of the actually returned cell range.
 ##
-## If one of the Java interfaces is used, field @var{xls}.limits contains
-## the outermost column and row numbers of the actually read cell range.
+## Optional return argument @var{rstatus} will be set to 1 if the
+## requested data have been read successfully, 0 otherwise. 
 ##
 ## Erroneous data and empty cells turn up empty in @var{rawarr}.
 ## Date/time values in Excel are returned as numerical values.
@@ -71,11 +84,10 @@
 ## 1/1/0000, resp.)
 ## Be aware that Excel trims @var{rawarr} from empty outer rows & columns, 
 ## so any returned cell array may turn out to be smaller than requested
-## in @var{range}.
-## When using COM or POI interface, formulas in cells are evaluated; if
-## that fails cached values are retrieved. These may be outdated 
-## depending on Excel's "Automatic calculation" settings when the
-## spreadsheet was saved.
+## in @var{range}, independent of field 'formulas_as_text' in @var{options}.
+## When using COM, POI, or UNO interface, formulas in cells are evaluated; if
+## that fails cached values are retrieved. These may be outdated depending
+## on Excel's "Automatic calculation" settings when the spreadsheet was saved.
 ##
 ## When reading from merged cells, all array elements NOT corresponding 
 ## to the leftmost or upper Excel cell will be treated as if the
@@ -97,7 +109,7 @@
 ##   [An, xls2, status] = xls2oct (xls2, 'Third_sheet');
 ## @end example
 ##
-## @seealso oct2xls, xlsopen, xlsclose, parsecell, xlsread, xlsfinfo, xlswrite 
+## @seealso {oct2xls, xlsopen, xlsclose, parsecell, xlsread, xlsfinfo, xlswrite }
 ##
 ## @end deftypefn
 
@@ -114,8 +126,15 @@
 ## 2010-11-07 More rigorous input checks. 
 ## 2010-11-12 Moved pointer check into main func
 ## 2010-11-13 Catch empty sheets when no range was specified
+## 2011-03-26 OpenXLS support added
+## 2011-03-29 Test for proper input xls struct extended
+## 2011-05-18 Experimental UNO support added
+## 2011-09-08 Minor code layout
+## 2012-01-26 Fixed "seealso" help string
+## 2012-02-25 Fixed missing quotes in struct check L.149-153
+## 2012-02-26 Updated texinfo header help text
 ##
-## Latest subfunc update: 2010-11-14 (poi)
+## Latest subfunc update: 2012-02-25
 
 function [ rawarr, xls, rstatus ] = xls2oct (xls, wsh=1, datrange='', spsh_opts=[])
 
@@ -125,9 +144,11 @@ function [ rawarr, xls, rstatus ] = xls2oct (xls, wsh=1, datrange='', spsh_opts=
 	test1 = test1 || ~isfield (xls, "workbook");
 	test1 = test1 || isempty (xls.workbook);
 	test1 = test1 || isempty (xls.app);
+	test1 = test1 || ~ischar (xls.xtype);
 	if test1
 		error ("Invalid xls file pointer struct");
 	endif
+
 	# Check worksheet ptr
 	if (~(ischar (wsh) || isnumeric (wsh))), error ("Integer (index) or text (wsh name) expected for arg # 2"); endif
 	# Check range
@@ -139,8 +160,8 @@ function [ rawarr, xls, rstatus ] = xls2oct (xls, wsh=1, datrange='', spsh_opts=
 		spsh_opts.strip_array = 1;
 		# Future options:
 	elseif (isstruct (spsh_opts))
-		if (~isfield (spsh_opts', formulas_as_text')), spsh_opts.formulas_as_text = 0; endif
-		if (~isfield (spsh_opts', strip_array')), spsh_opts.strip_array = 1; endif
+		if (~isfield (spsh_opts', 'formulas_as_text')), spsh_opts.formulas_as_text = 0; endif
+		if (~isfield (spsh_opts', 'strip_array')), spsh_opts.strip_array = 1; endif
 		% Future options:
 	else
 		error ("Structure expected for arg # 4");
@@ -157,6 +178,12 @@ function [ rawarr, xls, rstatus ] = xls2oct (xls, wsh=1, datrange='', spsh_opts=
 	elseif (strcmp (xls.xtype, 'JXL'))
 		# Read xls file tru JExcelAPI
 		[rawarr, xls, rstatus] = xls2jxla2oct (xls, wsh, datrange, spsh_opts);
+	elseif (strcmp (xls.xtype, 'OXS'))
+		# Read xls file tru OpenXLS
+		[rawarr, xls, rstatus] = xls2oxs2oct (xls, wsh, datrange, spsh_opts);
+	elseif (strcmp (xls.xtype, 'UNO'))
+		# Read xls file tru OpenOffice.org UNO (Java) bridge
+		[rawarr, xls, rstatus] = xls2uno2oct (xls, wsh, datrange, spsh_opts);
 #	elseif ---- <Other interfaces here>
 		# Call to next interface
 	else
@@ -190,7 +217,7 @@ endfunction
 
 
 #====================================================================================
-## Copyright (C) 2009,2010 P.R. Nienhuis, <pr.nienhuis at hccnet.nl>
+## Copyright (C) 2009,2010,2011,2012 P.R. Nienhuis, <pr.nienhuis at hccnet.nl>
 ##
 ## based on mat2xls by Michael Goffioul (2007) <michael.goffioul@swing.be>
 ##
@@ -223,7 +250,7 @@ endfunction
 ##   Arr = xls2com2oct (xls, 'Second_sheet');
 ## @end example
 ##
-## @seealso xls2oct, oct2xls, xlsopen, xlsclose, xlsread, xlswrite
+## @seealso {xls2oct, oct2xls, xlsopen, xlsclose, xlsread, xlswrite}
 ##
 ## @end deftypefn
 
@@ -238,6 +265,7 @@ endfunction
 ## 2010-10-24 Added check for "live" ActiveX server
 ## 2010-11-12 Moved ptr struct check into main func
 ## 2010-11-13 Catch empty sheets when no range was specified
+## 2012-01-26 Fixed "seealso" help string
 
 function [rawarr, xls, rstatus ] = xls2com2oct (xls, wsh, crange)
 
@@ -334,7 +362,7 @@ endfunction
 
 #==================================================================================
 
-## Copyright (C) 2009,2010 Philip Nienhuis <prnienhuis at users.sf.net>
+## Copyright (C) 2009,2010,2011,2012 Philip Nienhuis <prnienhuis at users.sf.net>
 ## 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -367,7 +395,7 @@ endfunction
 ##   B = xls2jpoi2oct (xls, 'Second_sheet', 'B3');
 ## @end example
 ##
-## @seealso xls2oct, oct2xls, xlsopen, xlsclose, xlsread, xlswrite, oct2jpoi2xls
+## @seealso {xls2oct, oct2xls, xlsopen, xlsclose, xlsread, xlswrite, oct2jpoi2xls}
 ##
 ## @end deftypefn
 
@@ -383,9 +411,10 @@ endfunction
 ## 2010-11-12 Moved ptr struct check into main func
 ## 2010-11-13 Catch empty sheets when no range was specified
 ## 2010-11-14 Fixed sheet # index (was offset by -1) in call to getusedrange() in case
-#3            of text sheet name arg
+##            of text sheet name arg
+## 2012-01-26 Fixed "seealso" help string
 
-function [ rawarr, xls, status ] = xls2jpoi2oct (xls, wsh, cellrange=[], spsh_opts)
+function [ rawarr, xls, rstatus ] = xls2jpoi2oct (xls, wsh, cellrange=[], spsh_opts)
 
 	persistent ctype;
 	if (isempty (ctype))
@@ -398,9 +427,9 @@ function [ rawarr, xls, status ] = xls2jpoi2oct (xls, wsh, cellrange=[], spsh_op
 		ctype(6) = java_get ('org.apache.poi.ss.usermodel.Cell', 'CELL_TYPE_ERROR');
 	endif
 	
-	status = 0; jerror = 0;
+	rstatus = 0; jerror = 0;
 	wb = xls.workbook;
-	
+
 	# Check if requested worksheet exists in the file & if so, get pointer
 	nr_of_sheets = wb.getNumberOfSheets ();
 	if (isnumeric (wsh))
@@ -457,7 +486,7 @@ function [ rawarr, xls, status ] = xls2jpoi2oct (xls, wsh, cellrange=[], spsh_op
 					type_of_cell = scell.getCellType ();
 					if (type_of_cell == ctype(3))        # Formula
 						if ~(spsh_opts.formulas_as_text)
-							try		# Because not al Excel formulas have been implemented
+							try		# Because not al Excel formulas have been implemented in POI
 								cvalue = frm_eval.evaluate (scell);
 								type_of_cell = cvalue.getCellType();
 								# Separate switch because form.eval. yields different type
@@ -505,14 +534,14 @@ function [ rawarr, xls, status ] = xls2jpoi2oct (xls, wsh, cellrange=[], spsh_op
 
 	if (jerror > 0) warning (sprintf ("xls2oct: %d cached values instead of formula evaluations.\n", jerror)); endif
 	
-	status = 1;
+	rstatus = 1;
 	xls.limits = [lcol, rcol; firstrow, lastrow];
 	
 endfunction
 
 
 #==================================================================================
-## Copyright (C) 2009 Philip Nienhuis <prnienhuis at users.sf.net>
+## Copyright (C) 2009,2010,2011,2012 Philip Nienhuis <prnienhuis at users.sf.net>
 ## 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -545,7 +574,7 @@ endfunction
 ##   B = xls2jxla2oct (xls, 'Second_sheet');
 ## @end example
 ##
-## @seealso xls2oct, oct2xls, xlsopen, xlsclose, xlsread, xlswrite, oct2jxla2xls
+## @seealso {xls2oct, oct2xls, xlsopen, xlsclose, xlsread, xlswrite, oct2jxla2xls}
 ##
 ## @end deftypefn
 
@@ -560,10 +589,13 @@ endfunction
 ##      "     calling function
 ## 2010-11-12 Moved ptr struct check into main func
 ## 2010-11-13 Catch empty sheets when no range was specified
+## 2011-04-11 (Ron Goldman <ron@ocean.org.il>) Fixed missing months var, wrong arg
+##      "     order in strsplit, wrong isTime condition
+## 2012-01-26 Fixed "seealso" help string
 
-function [ rawarr, xls, status ] = xls2jxla2oct (xls, wsh, cellrange=[], spsh_opts)
+function [ rawarr, xls, rstatus ] = xls2jxla2oct (xls, wsh, cellrange=[], spsh_opts)
 
-	persistent ctype;
+	persistent ctype; persistent months;
 	if (isempty (ctype))
 		ctype = cell (11, 1);
 		# Get enumerated cell types. Beware as they start at 0 not 1
@@ -578,9 +610,10 @@ function [ rawarr, xls, status ] = xls2jxla2oct (xls, wsh, cellrange=[], spsh_op
 		ctype( 9) = (java_get ('jxl.CellType', 'LABEL')).toString ();
 		ctype(10) = (java_get ('jxl.CellType', 'NUMBER_FORMULA')).toString ();
 		ctype(11) = (java_get ('jxl.CellType', 'STRING_FORMULA')).toString ();
+		months = {'JAN', 'FEB', 'MAR', 'APR', 'MAY', 'JUN', 'JUL', 'AUG', 'SEP', 'OCT', 'NOV', 'DEC'};
 	endif
 	
-	status = 0; 
+	rstatus = 0; 
 	wb = xls.workbook;
 	
 	# Check if requested worksheet exists in the file & if so, get pointer
@@ -588,7 +621,7 @@ function [ rawarr, xls, status ] = xls2jxla2oct (xls, wsh, cellrange=[], spsh_op
 	shnames = char (wb.getSheetNames ());
 	if (isnumeric (wsh))
 		if (wsh > nr_of_sheets), error (sprintf ("Worksheet # %d bigger than nr. of sheets (%d) in file %s", wsh, nr_of_sheets, xls.filename)); endif
-		sh = wb.getSheet (wsh - 1);			# POI sheet count 0-based
+		sh = wb.getSheet (wsh - 1);			# JXL sheet count 0-based
 		printf ("(Reading from worksheet %s)\n", shnames {wsh});
 	else
 		sh = wb.getSheet (wsh);
@@ -649,14 +682,14 @@ function [ rawarr, xls, status ] = xls2jxla2oct (xls, wsh, cellrange=[], spsh_op
 						rawarr {ii+1-firstrow, jj+1-lcol} = scell.getValue ();
 					catch
 						% Newer JXL.JAR, returns date string w. epoch = 1-1-1900 :-(
-						tmp = strsplit (' ', char (scell.getDate ()));
+						tmp = strsplit (char (scell.getDate ()), ' ');
 						yy = str2num (tmp{6});
 						mo = find (ismember (months, upper (tmp{2})) == 1);
 						dd = str2num (tmp{3});
 						hh = str2num (tmp{4}(1:2));
 						mi = str2num (tmp{4}(4:5));
 						ss = str2num (tmp{4}(7:8));
-						if (~scell.isTime ())
+						if (scell.isTime ())
 							yy = mo = dd = 0;
 						endif
 						rawarr {ii+1-firstrow, jj+1-lcol} = datenum (yy, mo, dd, hh, mi, ss);
@@ -680,7 +713,7 @@ function [ rawarr, xls, status ] = xls2jxla2oct (xls, wsh, cellrange=[], spsh_op
 						unwind_protect_cleanup
 							if (isempty (rawarr {ii+1-firstrow, jj+1-lcol}))
 								% Newer JXL.JAR, returns date string w. epoch = 1-1-1900 :-(
-								tmp = strsplit (' ', char (scell.getDate ()));
+								tmp = strsplit (char (scell.getDate ()), ' ');
 								yy = str2num (tmp{6});
 								mo = find (ismember (months, upper (tmp{2})) == 1);
 								dd = str2num (tmp{3});
@@ -720,7 +753,241 @@ function [ rawarr, xls, status ] = xls2jxla2oct (xls, wsh, cellrange=[], spsh_op
 		endfor
 	endfor
 
-	status = 1;
+	rstatus = 1;
 	xls.limits = [lcol, rcol; firstrow, lastrow];
 	
 endfunction
+
+
+## Copyright (C) 2011 Philip Nienhuis <prnienhuis at users.sf.net>
+## 
+## This program is free software; you can redistribute it and/or modify
+## it under the terms of the GNU General Public License as published by
+## the Free Software Foundation; either version 2 of the License, or
+## (at your option) any later version.
+## 
+## This program is distributed in the hope that it will be useful,
+## but WITHOUT ANY WARRANTY; without even the implied warranty of
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
+## 
+## You should have received a copy of the GNU General Public License
+## along with Octave; see the file COPYING.  If not, see
+## <http://www.gnu.org/licenses/>.
+
+## -*- texinfo -*-
+## @deftypefn {Function File} [@var{obj}, @var{rstatus}, @var{xls} ] = xls2oxs2oct (@var{xls})
+## @deftypefnx {Function File} [@var{obj}, @var{rstatus}, @var{xls} ] = xls2oxs2oct (@var{xls}, @var{wsh})
+## @deftypefnx {Function File} [@var{obj}, @var{rstatus}, @var{xls} ] = xls2oxs2oct (@var{xls}, @var{wsh}, @var{range})
+## Get cell contents in @var{range} in worksheet @var{wsh} in an Excel
+## file pointed to in struct @var{xls} into the cell array @var{obj}.
+## @var{range} can be a range or just the top left cell of the range.
+##
+## xls2oxs2oct should not be invoked directly but rather through xls2oct.
+##
+
+## Author: Philip Nienhuis
+## Created: 2011-03-26
+## Updates:
+## 2012-02-25 Changed ctype into num array rather than cell array
+
+function [ rawarr, xls, rstatus ] = xls2oxs2oct (xls, wsh, cellrange=[], spsh_opts)
+
+	persistent ctype;
+	if (isempty (ctype))
+		ctype = zeros (6, 1);
+		# Get enumerated cell types. Beware as they start at 0 not 1
+		ctype( 1) = (java_get ('com.extentech.ExtenXLS.CellHandle', 'TYPE_STRING'));  # 0
+		ctype( 2) = (java_get ('com.extentech.ExtenXLS.CellHandle', 'TYPE_FP'));      # 1
+		ctype( 3) = (java_get ('com.extentech.ExtenXLS.CellHandle', 'TYPE_INT'));     # 2
+		ctype( 4) = (java_get ('com.extentech.ExtenXLS.CellHandle', 'TYPE_FORMULA')); # 3
+		ctype( 5) = (java_get ('com.extentech.ExtenXLS.CellHandle', 'TYPE_BOOLEAN')); # 4
+		ctype( 6) = (java_get ('com.extentech.ExtenXLS.CellHandle', 'TYPE_DOUBLE'));  # 5
+	endif
+	
+	rstatus = 0; 
+	wb = xls.workbook;
+	
+	# Check if requested worksheet exists in the file & if so, get pointer
+	nr_of_sheets = wb.getNumWorkSheets ();
+	if (isnumeric (wsh))
+		if (wsh > nr_of_sheets), error (sprintf ("Worksheet # %d bigger than nr. of sheets (%d) in file %s", wsh, nr_of_sheets, xls.filename)); endif
+		sh = wb.getWorkSheet (wsh - 1);			# OXS sheet count 0-based
+		printf ("(Reading from worksheet %s)\n", sh.getSheetName ());
+	else
+		try
+			sh = wb.getWorkSheet (wsh);
+		catch
+			error (sprintf ("Worksheet %s not found in file %s", wsh, xls.filename));
+		end_try_catch
+	end
+
+	if (isempty (cellrange))
+		# Get numeric sheet pointer (0-based)
+		wsh = sh.getTabIndex ();
+		# Get data rectangle row & column numbers (1-based)
+		[firstrow, lastrow, lcol, rcol] = getusedrange (xls, wsh+1);
+		if (firstrow == 0 && lastrow == 0)
+			# Empty sheet
+			rawarr = {};
+			printf ("Worksheet '%s' contains no data\n", shnames {wsh});
+			rstatus = 1;
+			return;
+		else
+			nrows = lastrow - firstrow + 1;
+			ncols = rcol - lcol + 1;
+		endif
+	else
+		# Translate range to row & column numbers (1-based)
+		[dummy, nrows, ncols, firstrow, lcol] = parse_sp_range (cellrange);
+		# Check for too large requested range against actually present range
+		lastrow = min (firstrow + nrows - 1, sh.getLastRow + 1 ());
+		nrows = min (nrows, sh.getLastRow () - firstrow + 1);
+		ncols = min (ncols, sh.getLastCol () - lcol + 1);
+		rcol = lcol + ncols - 1;
+	endif
+
+	# Read contents into rawarr
+	rawarr = cell (nrows, ncols);			# create placeholder
+	for jj = lcol:rcol
+		for ii = firstrow:lastrow
+			try
+				scell = sh.getCell (ii-1, jj-1);
+				sctype = scell.getCellType ();
+				rawarr {ii+1-firstrow, jj+1-lcol} = scell.getVal ();
+				if (sctype == ctype(2) || sctype == ctype(3) || sctype == ctype(6))
+					rawarr {ii+1-firstrow, jj+1-lcol} = scell.getDoubleVal ();
+				endif
+			catch
+				# Empty or non-existing cell
+			end_try_catch
+		endfor
+	endfor
+
+	rstatus = 1;
+	xls.limits = [lcol, rcol; firstrow, lastrow];
+	
+endfunction
+
+
+## Copyright (C) 2011 Philip Nienhuis <prnienhuis@users.sf.net>
+## 
+## This program is free software; you can redistribute it and/or modify
+## it under the terms of the GNU General Public License as published by
+## the Free Software Foundation; either version 2 of the License, or
+## (at your option) any later version.
+## 
+## This program is distributed in the hope that it will be useful,
+## but WITHOUT ANY WARRANTY; without even the implied warranty of
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
+## 
+## You should have received a copy of the GNU General Public License
+## along with Octave; see the file COPYING.  If not, see
+## <http://www.gnu.org/licenses/>.
+
+## xls2uno2oct
+
+## Author: Philip Nienhuis <prnienhuis@users.sf.net>
+## Created: 2011-05-05
+## Updates:
+## 2011-09-18 Adapted sh_names type to LO 3.4.1
+## 2011-09-19 Try to decipher if formulas return numeric or string values
+
+function [rawarr, xls, rstatus] = xls2uno2oct  (xls, wsh, datrange, spsh_opts)
+
+  sheets = xls.workbook.getSheets ();
+  sh_names = sheets.getElementNames ();
+  if (! iscell (sh_names))
+    # Java array (LibreOffice 3.4.+); convert to cellstr
+    sh_names = char (sh_names);
+  else
+    sh_names = {sh_names};
+  endif
+
+  # Check sheet pointer
+  if (isnumeric (wsh))
+	  if (wsh < 1 || wsh > numel (sh_names))
+      error ("Sheet index %d out of range 1-%d", wsh, numel (sh_names));
+    endif
+  else
+    ii = strmatch (wsh, sh_names);
+    if (isempty (ii)), error ("Sheet '%s' not found", wsh); endif
+    wsh = ii;
+  endif
+  unotmp = java_new ('com.sun.star.uno.Type', 'com.sun.star.sheet.XSpreadsheet');
+  sh = sheets.getByName(sh_names{wsh}).getObject.queryInterface (unotmp);
+
+  unotmp = java_new ('com.sun.star.uno.Type', 'com.sun.star.sheet.XCellRangesQuery');
+  xRQ = sh.queryInterface (unotmp);
+  # Get cell ranges of all rectangles containing data. Type values:
+  #java_get ('com.sun.star.sheet.CellFlags', 'VALUE')  ans =  1
+  #java_get ('com.sun.star.sheet.CellFlags', 'DATETIME')  ans =  2
+  #java_get ('com.sun.star.sheet.CellFlags', 'STRING')  ans =  4
+  #java_get ('com.sun.star.sheet.CellFlags', 'FORMULA')  ans =  16
+  # Yep, boolean is lacking...
+  Cellflgs = javaObject ("java.lang.Short", "23");
+  ccells = xRQ.queryContentCells (Cellflgs);
+  addrs = ccells.getRangeAddressesAsString ();
+
+  # Strip sheet name from addresses
+  adrblks = strsplit (addrs, ',');
+  if (isempty (adrblks))
+    warning ('Sheet %s contains no data', sh_names{wsh});
+    return
+  endif
+
+  # Either parse (given cell range) or prepare (unknown range) help variables.
+  # As OpenOffice knows the occupied range, we need the limits anyway to avoid
+  # out-of-range errors
+  [ trow, brow, lcol, rcol ] = getusedrange (xls, wsh);
+  if (isempty (datrange))
+    nrows = brow - trow + 1;	# Number of rows to be read
+    ncols = rcol - lcol + 1;	# Number of columns to be read
+  else
+    [dummy, nrows, ncols, srow, scol] = parse_sp_range (datrange);
+    # Truncate range silently if needed
+    brow = min (srow + nrows - 1, brow);
+    rcol = min (scol + ncols - 1, rcol);
+    trow = max (trow, srow);
+    lcol = max (lcol, scol);
+    nrows = min (brow - trow + 1, nrows);	# Number of rows to be read
+    ncols = min (rcol - lcol + 1, ncols);	# Number of columns to be read
+  endif
+  # Create storage for data at Octave side
+  rawarr = cell (nrows, ncols);
+
+  # Get data. Apparently row & column indices are 0-based in UNO
+  for ii=trow-1:brow-1
+    for jj=lcol-1:rcol-1
+      XCell = sh.getCellByPosition (jj, ii);
+      cType = XCell.getType().getValue ();
+      switch cType
+        case 1	# Value
+          rawarr{ii-trow+2, jj-lcol+2} = XCell.getValue ();
+        case 2	# String
+          unotmp = java_new ('com.sun.star.uno.Type', 'com.sun.star.text.XText');
+          rawarr{ii-trow+2, jj-lcol+2} = XCell.queryInterface (unotmp).getString ();
+        case 3	# Formula
+          if (spsh_opts.formulas_as_text)
+            rawarr{ii-trow+2, jj-lcol+2} = XCell.getFormula ();
+          else
+            # Unfortunately OOo gives no clue as to the type of formula result
+            unotmp = java_new ('com.sun.star.uno.Type', 'com.sun.star.text.XText');
+            rawarr{ii-trow+2, jj-lcol+2} = XCell.queryInterface (unotmp).getString ();
+            tmp = str2double (rawarr{ii-trow+2, jj-lcol+2});
+            # If the string happens to contain just a number we'll assume it is numeric
+            if (~isnan (tmp)); rawarr{ii-trow+2, jj-lcol+2} = tmp; endif
+          endif
+        otherwise
+          # Empty cell
+      endswitch
+    endfor
+  endfor 
+
+  # Keep track of data rectangle limits
+  xls.limits = [lcol, rcol; trow, brow];
+
+  rstatus = ~isempty (rawarr);
+
+endfunction
