@@ -1,4 +1,4 @@
-## Copyright (C) 2009,2010 Philip Nienhuis <prnienhuis at users.sf.net>
+## Copyright (C) 2009,2010,2011,2012 Philip Nienhuis <prnienhuis at users.sf.net>
 ## 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -62,9 +62,12 @@
 ##
 ## The optional last argument @var{reqintf} can be used to override 
 ## the automatic selection by xlswrite of one interface out of the
-## supported ones: COM/Excel, Java/Apache POI, or Java/JExcelAPI.
-## For writing to OOXML files (.xlsx) a value of 'poi' (case-insensitive)
-## must be specified for @var{reqintf}.
+## supported ones: 'com' (ActiveX/Excel), 'poi' (Java/Apache POI), 'jxl'
+## (Java/JExcelAPI), or 'uno' (Java/OpenOffice.org). 'oxs' (Java/OpenXLS)
+## is implemented but disabled for writing. For writing to OOXML files
+## (.xlsx) a value of 'poi' or 'uno' must be specified for @var{reqintf}.
+## The value of @var{reqintf} is case-insensitive. Multiple interfaces
+## can be selected if entered as a cell array of strings.
 ##
 ## xlswrite is a mere wrapper for various scripts which find out what
 ## Excel interface to use (COM, Java/POI) plus code to mimic the other
@@ -83,7 +86,7 @@
 ##   True (= numerical 1) in status if al went well) 
 ## @end example
 ##
-## @seealso xlsread, oct2xls, xls2oct, xlsopen, xlsclose, xlsfinfo
+## @seealso {xlsread, oct2xls, xls2oct, xlsopen, xlsclose, xlsfinfo}
 ##
 ## @end deftypefn
 
@@ -95,7 +98,9 @@
 ## 2010-01-15 Fixed typos in texinfo
 ## 2010-08-18 Added check for existence of xls after call to xlsopen to 
 ##	          avoid unneeded error message clutter
-## 2010-10-27 Chnged range -> crange to unhide other range functions
+## 2010-10-27 Changed range -> crange to unhide other range functions
+## 2011-09-08 Minor code syntax updates
+## 2012-01-26 Fixed "seealso" help string
 
 function [ rstatus ] = xlswrite (filename, arr, arg3, arg4, arg5)
 
@@ -153,14 +158,14 @@ function [ rstatus ] = xlswrite (filename, arr, arg3, arg4, arg5)
 	endif
 
 	unwind_protect				# Needed to be sure Excel can be closed i.c.o. errors
-	xls_ok = 0;
-	xls = xlsopen (filename, 1, reqintf);
-	xls_ok = 1;
+    xls_ok = 0;
+    xls = xlsopen (filename, 1, reqintf);
+    xls_ok = 1;
 
-	[xls, rstatus] = oct2xls (arr(1:nr, 1:nc), xls, wsh, topleft);
+    [xls, rstatus] = oct2xls (arr(1:nr, 1:nc), xls, wsh, topleft);
 
 	unwind_protect_cleanup
-	if (xls_ok) xls = xlsclose (xls); endif
+    if (xls_ok), xls = xlsclose (xls); endif
 
 	end_unwind_protect
 
